#
# Cookbook Name:: apt
# Recipe:: default
#
# Copyright 2008-2009, Opscode, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
log 'PLATAFORM....' do
  message node["platform"] + " - " + node["platform_version"]
  level :info
end


bash "add datastax keys" do
  user "root"
  code <<-EOC
   curl -L http://debian.datastax.com/debian/repo_key | apt-key add -
  EOC
end

execute "apt-get update" do
  action :nothing
end

template "/etc/apt/apt.conf" do
  source  "apt.conf.erb"
  group   "root"
  owner   "root"
  mode    0755
  action  :create
  only_if { node.apt != nil && node.apt.cache_server != nil && node.apt.cache_server != "" }
  notifies :run, resources(:execute => "apt-get update"), :immediately
end

# When no cache_Server is specified, remove the apt.conf
# TODO: should perform a sed and remove the Acquire::http::Proxy line only
file "/etc/apt/apt.conf" do
  action  :delete
  only_if { node.apt != nil && node.apt.cache_server != nil && node.apt.cache_server == "" }
end

%w{/var/cache/local /var/cache/local/preseeding}.each do |dirname|
  directory dirname do
    owner "root"
    group "root"
    mode  0755
    action :create
  end
end


node.inconcert.sources.repourl = Inconcert::GenericHelpers::packages_repo_url(node)
log 'PLATAFORM....' do
  message "Llego hasta aca"
  level :info
end


if node["platform"] == "debian" then
	if node["platform_version"].start_with? "6" then
		source_repo = "sources.list.squeeze.erb"
		source = "i6-squeeze.list.erb"
	else	
		if node["platform_version"].start_with? "8" then
			if node.inconcert.repo_pack_debian_8_url == nil then
				source_repo = "sources.list.jessie.empty.erb"
			else
				if node.inconcert.repo_pack_debian_8_url == "" then
					source_repo = "sources.list.jessie.empty.erb"
				else
					source_repo = "sources.list.jessie.erb"
				end
			end
			source = "i6-jessie.list.erb"
		end
	end
else
	if node["platform"] == "ubuntu" then
		if node["platform_version"].start_with? "18" then
			if node.inconcert.repo_pack_ubuntu_bionic_url == nil then
				source_repo = "sources.list.ubuntu.bionic.empty.erb"
			else
				if node.inconcert.repo_pack_ubuntu_bionic_url == "" then
					source_repo = "sources.list.ubuntu.bionic.empty.erb"
				else
					source_repo = "sources.list.ubuntu.bionic.erb"
				end
			end
			source = "i6-ubuntu.bionic.list.erb"
		end
		if node["platform_version"].start_with? "22" then
			log 'PLATAFORM....' do
			  message node.inconcert.repo_pack_ubuntu_jammy_url
			  level :info
			end
			if node.inconcert.repo_pack_ubuntu_jammy_url == nil then
				source_repo = "sources.list.ubuntu.jammy.empty.erb"
			else
				if node.inconcert.repo_pack_ubuntu_jammy_url == "" then
					source_repo = "sources.list.ubuntu.jammy.empty.erb"
				else
					source_repo = "sources.list.ubuntu.jammy.erb"
				end
			end
			source = "i6-ubuntu.jammy.list.erb"
		end
	else
		source = "i6.list.erb"
		source_repo = "sources.list.jessie.empty.erb"
	end
end

template "/etc/apt/sources.list" do
  source source_repo
  owner  "root"
  group  "root"
  mode   "0644"
  notifies :run, resources(:execute => "apt-get update"), :immediately
end

template "/etc/apt/sources.list.d/inconcert6.list" do
  source source
  owner  "root"
  group  "root"
  mode   "0644"
  notifies :run, resources(:execute => "apt-get update"), :immediately
end


cookbook_file "/etc/apt/sources.list.d/backports.list" do
  source  "backports.list"
  owner   "root"
  group   "root"
  mode    "0755"
  notifies :run, resources(:execute => "apt-get update"), :immediately
  only_if { node.platform_version.start_with? "5.0" }
end

cookbook_file "/etc/apt/apt.conf.d/20auto-upgrades" do
  source  "20auto-upgrades"
  owner   "root"
  group   "root"
  mode    "0755"
  only_if { node["platform"] == "ubuntu" }
end



