#
# Cookbook Name:: asterisk
# Recipe:: default
#
# Copyright 2012, inConcert
#
# All rights reserved - Do Not Redistribute
#

include_recipe "apt"

execute "packages update check" do
  command "ls"
  notifies :run, resources(:execute => "apt-get update"), :immediately
end

package "icc-codecs" do
	# force yes is a hack because I do not have a signed repo.
	options '--reinstall --force-yes -o Dpkg::Options::="--force-confnew"'
	action :nothing
end

package "icc-asterisk" do
	# force yes is a hack because I do not have a signed repo.
	options '--force-yes -o Dpkg::Options::="--force-confnew"'
	version node.inconcert.packages.icc_asterisk if node.inconcert.packages.icc_asterisk != "*"
	notifies :remove, resources(:package => "icc-codecs"), :immediately
	notifies :install, resources(:package => "icc-codecs"), :immediately
end

bash "bootstrap asterisk" do
  cwd   "/usr/share/inconcert/asterisk"
  code  <<-EOC
    ./bootstrap.sh
  EOC
end

# This is to contemplate installations where we already have Asterisk installed, but no codecs
package "icc-codecs" do
    # force yes is a hack because I do not have a signed repo.
    options '--force-yes -o Dpkg::Options::="--force-confnew"'
end

bash "Create Directories for logrotate" do
  user "root"
  code <<-EOC 

    # break on first error
    set -e
    mkdir -p /etc/logrotate_asterisk.d
  EOC
end

cookbook_file "/etc/logrotate.d/asterisk" do
  source "logrotate"
  owner "root"
  mode "0444"
end

cookbook_file "/etc/logrotate_asterisk.d/asterisk" do
  source "logrotate"
  owner "root"
  mode "0444"
end

cookbook_file "/etc/logrotate_onlyAsterisk.conf" do
  source "logrotate_onlyAsterisk.conf"
  owner "root"
  mode "0444"
end

# run cores cleaner hourly
cookbook_file "/usr/sbin/icc_logrotate.sh" do
  source "icc_logrotate.sh"
  owner "root"
  group "root"
  mode  0744
end

bash "add logrotate_onlyAsterisk to cron" do
  user "root"
  code <<-EOC 

    # break on first error
    set -e
	
    grep -q -F '*/5 * * * * root /usr/sbin/icc_logrotate.sh' /etc/crontab || echo '*/5 * * * * root /usr/sbin/icc_logrotate.sh' >> /etc/crontab
    
	/etc/init.d/cron restart

  EOC
#  not_if "ls /usr/share/inconcert/helpers/inconcert_cleanupdata/.bootstrapped"
end

#Logger Asterisk
cookbook_file "/etc/asterisk/logger.conf" do
  source "logger.conf"
  owner "root"
  group "root"
end

# Configuro HTTP.conf
template "/etc/asterisk/http.conf" do
  source "http.conf.erb"
  owner  "root"
  group  "root"
  mode   "0644"
end

# Configuro rtp.conf
template "/etc/asterisk/rtp.conf" do
  source "rtp.conf.erb"
  owner  "root"
  group  "root"
  mode   "0644"
end


bash "Creo carpeta ssl_certificates" do
  user "root"
  code <<-EOC
    mkdir -p "#{node.asterisk.prefix}/ssl_certificates"
  EOC
end

cookbook_file "#{node.asterisk.prefix}/ssl_certificates/server.key" do
  source "server.key"
  mode "0644"
end

cookbook_file "#{node.asterisk.prefix}/ssl_certificates/server.pem" do
  source "server.pem"
  mode "0644"
end

#package "ffmpeg" do
#	# force yes is a hack because I do not have a signed repo.
#  	options "--force-yes"
#  	# install explicit version on Debian Squeeze
#  	version "5:0.7.16-dmo1" if (node["platform"] == "debian" and node["platform_version"].start_with? "6")
#end


# Instalo paquetes para UniMRCP
bash "Instaling unimrcp-deps" do
	user    "root"
	cwd     "/usr/src/asterisk"
	code  <<-EOH
		wget -q http://#{node.inconcert.sources.server}/repo/unimrcp-deps-1.6.0.tar.gz -O /usr/src/asterisk/unimrcp-deps-1.6.0.tar.gz
		cd /usr/src/asterisk
		rm -rf unimrcp-deps-1.6.0
		tar -xvzf unimrcp-deps-1.6.0.tar.gz
		cd unimrcp-deps-1.6.0/
		./build-dep-libs.sh --silent
		rm -rf /lib/libsofia-sip-ua.so.0
		ln -s /usr/local/lib/libsofia-sip-ua.so.0.6.0 /lib/libsofia-sip-ua.so.0
	EOH
end

bash "Instaling unimrcp" do
	user    "root"
	cwd     "/usr/src/asterisk"
	code  <<-EOH
		wget -q http://#{node.inconcert.sources.server}/repo/unimrcp-unimrcp-1.7.0.tar.gz -O /usr/src/asterisk/unimrcp-unimrcp-1.7.0.tar.gz
		cd /usr/src/asterisk
		rm -rf unimrcp-unimrcp-1.7.0
		tar -xvzf unimrcp-unimrcp-1.7.0.tar.gz
		cd unimrcp-unimrcp-1.7.0
		make clean
		make distclean
		./bootstrap
		./configure
		make
		make install
	EOH
end

bash "asterisk-unimrcp" do
	user    "root"
	cwd     "/usr/src/asterisk"
	code  <<-EOH
		wget -q http://#{node.inconcert.sources.server}/repo/asterisk-unimrcp-asterisk-unimrcp-1.7.0.tar.gz -O /usr/src/asterisk/asterisk-unimrcp-asterisk-unimrcp-1.7.0.tar.gz
		cd /usr/src/asterisk
		rm -rf asterisk-unimrcp-asterisk-unimrcp-1.7.0
		tar -xvzf asterisk-unimrcp-asterisk-unimrcp-1.7.0.tar.gz
		cd asterisk-unimrcp-asterisk-unimrcp-1.7.0
		./bootstrap
		./configure
		make
		make install
	EOH
end