
#
# Cookbook Name:: cassandra
# Recipe:: default
#
# Copyright 2012, inConcert
#
# All rights reserved - Do Not Redistribute
#

#
# make CassandraHelpers methods available inside bash script resources.
#
# FIXME probar si anda mover esta declaración al library
#
class Erubis::Context
  include CassandraHelpers
end

if node["platform"] == "ubuntu" and node["platform_version"].start_with? "22" then
	package "python-minimal" do
	end
	
	package "python2"do
		not_if "ls /usr/share/inconcert/cassandra/.bootstrapped"
	end
	
	package "libpython-stdlib"do
		not_if "ls /usr/share/inconcert/cassandra/.bootstrapped"
	end
	
	package "python"do
		not_if "ls /usr/share/inconcert/cassandra/.bootstrapped"
	end
	
	package "libffi6"do
		not_if "ls /usr/share/inconcert/cassandra/.bootstrapped"
	end
	
	package "libjna-jni" do
		version "4.5.1-1"
	end
	
	package "libjna-java" do
		version "4.5.1-1"
	end
end
#
#  FIXME define service so we can notify in case of config changes
#  FIXME notify in case of config changes
#

include_recipe "apt"

package "curl" do
	options '--force-yes'
end

cookbook_file "/etc/apt/sources.list.d/cassandra.list" do
  source "cassandra.list"
end

# Pin Cassandra version at 1.2.13
cookbook_file "/etc/apt/preferences.d/pin-cassandra" do
  source "pin-cassandra"
  mode "644"
end

bash "add datastax keys" do
  user "root"
  code <<-EOC
   curl -L http://debian.datastax.com/debian/repo_key | apt-key add - 
  EOC
  #not_if "apt-key list | grep Riptano"
  notifies :run, resources(:execute => "apt-get update"), :immediately
end

package "cassandra" do
  version node.cassandra.version
  options '--allow-unauthenticated --allow-downgrades --allow-remove-essential --allow-change-held-packages -o Dpkg::Options::="--force-confold" -o Dpkg::Options::="--force-confmiss"'   # this is a hack because I do not have a signed repo.
  ignore_failure true
end

cookbook_file "/etc/init.d/cassandra" do
  source "cassandra"
  owner   "root"
  group   "root"
  mode    "0755"
  only_if { node["platform"] == "ubuntu" }
end

bash "systemctl reload" do 
  user "root"
  code <<-EOC
	systemctl daemon-reload
  EOC
  only_if { node["platform"] == "ubuntu" }
end

bash "restart cassandra" do 
  user "root"
  code <<-EOC
	/etc/init.d/cassandra stop
	sleep 5
    /etc/init.d/cassandra start
	sleep 5
    for i in {1..150}
    do
       echo "Check status of cassandra $i"
       tail -n 10 /var/log/cassandra/system.log| grep "istening for thrift clients..." && break
       sleep 2
    done
    tail -n 10 /var/log/cassandra/system.log| grep "istening for thrift clients..." || exit 13
  EOC
  action :nothing
end

bash "remove jdk-7-icc" do
  user "root"
  code <<-EOC
    apt-get remove jdk-7-icc --assume-yes
  EOC
  ignore_failure true
end

package "jdk-7-icc" do
  options '-o Dpkg::Options::="--force-confold" -o Dpkg::Options::="--force-confmiss"'   # this is a hack because I do not have a signed repo.
  notifies :run, resources( :bash => "restart cassandra" ), :immediately
  ignore_failure true
end

template "/etc/cassandra/cassandra.yaml" do 
  source "cassandra.yaml.erb"
  owner  "root"
  group  "root"
  mode   "0644"
  notifies :run, resources( :bash => "restart cassandra" )
end

directory "/etc/prometheus_exporters/jmx_exporter" do
  recursive true
  owner  "root"
  group  "root"
  mode   "0700"
  action :create
end

bash "Download and install jmx_exporter" do
	user    "root"
	cwd     "/etc/prometheus_exporters/jmx_exporter"
	code  <<-EOH
		wget -q http://#{node.inconcert.sources.server}/repo/jmx-exporter_cassandra.gz -O /etc/prometheus_exporters/jmx_exporter/jmx-exporter_cassandra.gz
		cd /etc/prometheus_exporters/jmx_exporter
		tar -xvzf jmx-exporter_cassandra.gz
	EOH
end

template "/etc/cassandra/cassandra-env.sh" do 
  source "cassandra-env.sh.erb"
  owner  "root"
  group  "root"
  mode   "0644"
  notifies :run, resources(:bash => "restart cassandra")
end

template "/etc/cassandra/cassandra-rackdc.properties" do 
  source "cassandra-rackdc.properties.erb"
  owner  "root"
  group  "root"
  mode   "0644"
  notifies :run, resources(:bash => "restart cassandra")
end

cookbook_file "/etc/cassandra/cassandra-topology.properties" do
  source "cassandra-topology.properties"
end

node.cassandra.data_file_directories.each do |d| 
  directory d do
    recursive true
    owner  "cassandra"
    group  "cassandra"
    mode   "0700"
    action :create
  end
end

directory "#{node.cassandra.commitlog_directory}" do
  recursive true
  owner  "cassandra"
  group  "cassandra"
  mode   "0700"
  action :create
end

link "link to jna.jar" do
  link_type :symbolic
  owner "root"
  group "root"
  target_file "#{node.cassandra.lib_path}"
  to "/usr/share/java/jna.jar"
  action :create
end



#
# If this is the first time, we need to destroy and rebuild
# because on install the package starts cassandra creating a 
# data dir associated with the default cluster name.
#
# A possible fix is patching the package to _not_ start the server on install.
# This fix is also interesting if we patch it to use runit. (2x1!)
#
#
backupdir = "/var/backup/cassandra-#{Time.new.to_i}"
bash "rebuild cassandra" do
  user "root"
  code <<-EOC
    /etc/init.d/cassandra stop
	sleep 10
	ps -ax | grep "jsvc.exec -user cassandra" | grep -v "grep" | awk '{ print $1 }' | xargs kill -9 
    mkdir -p #{backupdir}
    mv /var/lib/cassandra/* #{backupdir}/
    /etc/init.d/cassandra start
    for i in {1..60}
    do
       echo "Check $i"
       tail -n 10 /var/log/cassandra/system.log| grep "istening for thrift clients..." && break
       sleep 2
    done
    tail -n 10 /var/log/cassandra/system.log| grep "istening for thrift clients..." || exit 13
    touch /var/lib/cassandra/.bootstraped
  EOC
  not_if "ls /var/lib/cassandra/.bootstraped"
end



# remove previous hotfixes
cookbook_file "/removeOldHotfix.sh" do
	source "removeOldHotfix.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/removeOldHotfix.lua" do
	source "removeOldHotfix.lua"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/GetHotfixesInstalados.lua" do
	source "GetHotfixesInstalados.lua"
	owner   "root"
	group   "root"
	mode    "0744"
end

bash "Check previus Hotfixes" do
	user    "root"
	cwd     "/tmp"
	code  <<-EOH
    echo "========================================================================================================"
    echo "                                      Check hotxies"
    echo "========================================================================================================"
    bash /removeOldHotfix.sh #{node.inconcert.packages.inconcert.split("+")[0]}
    echo "========================================================================================================"
    rm /removeOldHotfix.sh
    rm /removeOldHotfix.lua
    rm /GetHotfixesInstalados.lua
    rm /host.txt
	EOH
end