#
# Cookbook Name:: elasticSearch
# Recipe:: default
#
# Copyright 2012, inConcert
#
# All rights reserved - Do Not Redistribute
#


#
# make helper methods available inside templates.
#
#class Erubis::Context
#	include Redis::Helpers
#end

include_recipe "runit"

if node.inconcert.elasticsearch_nodes.size == 0 then
	raise "Debe ingresar al menos un nodo para elasticsearch"
end 

esNode = false
esDialerNode = false
node.elasticsearch.number_of_shards = "1"
if node.inconcert.elasticsearch_nodes.size == 1 then
	node.elasticsearch.number_of_replicas = "0"
else
	if node.inconcert.elasticsearch_nodes.size == 2 then
		node.elasticsearch.number_of_replicas = "1"
	else
		node.elasticsearch.number_of_replicas = "2"
	end
end

node.elasticsearch_dialer.number_of_shards = "1"
if node.inconcert.elasticsearch_dialer_nodes.size == 1 then
	node.elasticsearch_dialer.number_of_replicas = "0"
else
	if node.inconcert.elasticsearch_dialer_nodes.size == 2 then
		node.elasticsearch_dialer.number_of_replicas = "1"
	else
		node.elasticsearch_dialer.number_of_replicas = "2"
	end
end

node.inconcert.elasticsearch_nodes.map do |n|
	if n['id'].is_a? String then
		if n['id'] == node.hostname then
			esNode = true
			if n['path_to_data'].size > 0 then
				node.elasticsearch.es_path_to_data = n['path_to_data']
			else
				raise "Debe seleccion el path para la data de ElasticSearch"
			end
			if n['path_to_log'].is_a? String then
				node.elasticsearch.es_path_to_log = n['path_to_log']
			else
				raise "Debe seleccion el path para los logs de ElasticSearch"
			end
			if n['private_ip'].is_a? String then
				node.elasticsearch.private_ip = n['private_ip']
			else	
				raise "Debe seleccion la IP privada del nodo."
			end
		end
	end
end

if node["platform"] == "ubuntu" then
	node.inconcert.elasticsearch_dialer_nodes.map do |n|
		if n['id'].is_a? String then
			if n['id'] == node.hostname then
				esDialerNode = true
				if node.inconcert.elasticsearch_dialer_nodes.size == 2 then
					raise "La cantidad de nodos para ES Dialer debe ser 1 o mayor a 2."
				end
				if n['path_to_data'].size > 0 then
					node.elasticsearch_dialer.es_path_to_data = n['path_to_data']
				else
					raise "Debe seleccion el path para la data de ElasticSearch Dialer"
				end
				if n['path_to_log'].is_a? String then
					node.elasticsearch_dialer.es_path_to_log = n['path_to_log']
				else
					raise "Debe seleccion el path para los logs de ElasticSearch Dialer"
				end
				if n['private_ip'].is_a? String then
					node.elasticsearch_dialer.private_ip = n['private_ip']
				else	
					raise "Debe seleccion la IP privada del nodo."
				end
				node.elasticsearch_dialer.number_of_shards = "1"
				if node.inconcert.elasticsearch_dialer_nodes.size == 1 then
					node.elasticsearch_dialer.number_of_replicas = "0"
				else
					node.elasticsearch_dialer.number_of_replicas = "2"
				end
			end
		end
	end
end

if esNode or esDialerNode then
	#Elimino version anterior 
	bash "[ELASTIC-SEARCH] Delete old ElasticSearch" do
	  user    "root"
	  cwd     "/tmp"
	  code  <<-EOH
		rm -rf /usr/share/elasticsearch/bin/*
		rm -rf /usr/share/elasticsearch/config/*
		rm -rf /usr/share/elasticsearch/lib/*
		rm -rf /usr/share/elasticsearch/plugins/*
	  EOH
	  not_if "ls /usr/share/elasticsearch/.bootstrapped_es-#{node.elasticsearch.version}"
	end
	
	# Descargo e Instalo elasticsearch.
	bash "[ELASTIC-SEARCH] Download elasticsearch-#{node.elasticsearch.version}.tar.gz" do
	  user    "root"
	  cwd     "/tmp"
	  code  <<-EOH
		wget -q http://#{node.inconcert.sources.server}/repo/elasticsearch-#{node.elasticsearch.version}.tar.gz -O /tmp/elasticsearch-#{node.elasticsearch.version}.tar.gz
	  EOH
	  not_if "ls /usr/share/elasticsearch/.bootstrapped_es-#{node.elasticsearch.version}"
	end
	
	bash "[ELASTIC-SEARCH] Install elasticsearch-#{node.elasticsearch.version} from gz file" do
	  user    "root"
	  cwd     "/tmp"
	  code  <<-EOH
		tar -xvzf elasticsearch-#{node.elasticsearch.version}.tar.gz
		mkdir /usr/share/elasticsearch
		cp -r elasticsearch-#{node.elasticsearch.version}/* /usr/share/elasticsearch/
		rm -f elasticsearch-#{node.elasticsearch.version}.tar.gz
	  EOH
	  not_if "ls /usr/share/elasticsearch/.bootstrapped_es-#{node.elasticsearch.version}"
	end
	
	bash "[ELASTIC-SEARCH] add user elasticsearch" do
	  cwd "/usr/share/elasticsearch"
	  code <<-EOC
		adduser elasticsearch --home /usr/share/elasticsearch --group --system --quiet || exit 1
		chown -R elasticsearch:elasticsearch /usr/share/elasticsearch
		chmod -R 775  /usr/share/elasticsearch
	  EOC
	end
end

# Elastic Search Interacciones ==================================================================
if esNode then
	node.elasticsearch.es_path_to_data.each do |d| 
	  directory d do
		recursive true
		owner  "elasticsearch"
		group  "elasticsearch"
		mode   "0700"
		action :create
	  end
	end

	directory "#{node.elasticsearch.es_path_to_log}" do
	  recursive true
	  owner  "elasticsearch"
	  group  "elasticsearch"
	  mode   "0700"
	  action :create
	end
	
	node.elasticsearch.es_path_to_data.each do |d| 
		bash "[ELASTIC-SEARCH] permission for data directory" do
			cwd "#{d}"
			code <<-EOC
				chown -R elasticsearch:elasticsearch #{d}
				chmod -R 775  #{d}
			EOC
		end
	end
	
	bash "[ELASTIC-SEARCH] permission for log directory" do
	  cwd "#{node.elasticsearch.es_path_to_log}"
	  code <<-EOC
		chown -R elasticsearch:elasticsearch #{node.elasticsearch.es_path_to_log}
		chmod -R 775  #{node.elasticsearch.es_path_to_log}
	  EOC
	end
	
	node.elasticsearch.es_path_to_data.each do |d| 
		bash "[ELASTIC-SEARCH] Delete Old data on: #{d}/#{node.inconcert.cluster_id}/" do
		  cwd "#{d}"
		  code <<-EOC
			rm -rf #{d}/#{node.inconcert.cluster_id}/*
		  EOC
		  not_if "ls /usr/share/elasticsearch/.bootstrapped_es-#{node.elasticsearch.version}"
		end
	end
	
	cookbook_file "/usr/share/elasticsearch/bin/elasticsearch" do
	  source "elasticsearch"
	  owner   "elasticsearch"
	  group   "elasticsearch"
	  mode    "0777"
	end
	
	cookbook_file "/usr/share/elasticsearch/config/jvm.options" do
	  source "jvm.options"
	  owner   "elasticsearch"
	  group   "elasticsearch"
	  mode    "0644"
	end
	
	if node.inconcert.elasticsearch_nodes.size == 1 then
		template "/usr/share/elasticsearch/config/elasticsearch.yml" do 
		  source "elasticsearch.yml-single_node.erb"
		  owner  "elasticsearch"
		  group  "elasticsearch"
		  mode   "0644"
		end
	else
		if node.inconcert.elasticsearch_nodes.size == 2 then
			if node.inconcert.elasticsearch_ip == node.elasticsearch.private_ip then
				template "/usr/share/elasticsearch/config/elasticsearch.yml" do 
				  source "elasticsearch.yml-two_nodes_master.erb"
				  owner  "elasticsearch"
				  group  "elasticsearch"
				  mode   "0644"
				end
			else
				template "/usr/share/elasticsearch/config/elasticsearch.yml" do 
				  source "elasticsearch.yml-two_nodes_slave.erb"
				  owner  "elasticsearch"
				  group  "elasticsearch"
				  mode   "0644"
				end
			end
		else
			template "/usr/share/elasticsearch/config/elasticsearch.yml" do 
			  source "elasticsearch.yml-cluster.erb"
			  owner  "elasticsearch"
			  group  "elasticsearch"
			  mode   "0644"
			end
		end
	end
	
	dir_service="/etc/inconcert/sv/elasticsearch"
	if not Dir.exist?(dir_service) then
		bash "[ELASTIC-SEARCH] Create Elasticsearch service directory" do
		  user    "root"
		  cwd     "/root"
		  code  <<-EOH
			mkdir -p /etc/inconcert/sv/elasticsearch
		  EOH
		end
	end
	
	cookbook_file "/etc/inconcert/sv/elasticsearch/run" do
	  source "run"
	  owner   "root"
	  group   "root"
	  mode    "0777"
	end
	
	bash "[ELASTIC-SEARCH] Create Elasticsearch service" do
	  user    "root"
	  cwd     "/root"
	  code  <<-EOH
		ln -s /etc/inconcert/sv/elasticsearch /etc/service/elasticsearch
		/usr/sbin/update-service --add /etc/inconcert/sv/elasticsearch
	  EOH
	end
end

# Elastic Search Dialer ==================================================================
if esDialerNode and node["platform"] == "ubuntu" then
	bash "[ELASTIC-SEARCH-DIALER] Create Elasticsearch Config folder" do
	  user    "root"
	  cwd     "/root"
	  code  <<-EOH
		mkdir -p /usr/share/elasticsearch/config_dialer
		cp -r /usr/share/elasticsearch/config/* /usr/share/elasticsearch/config_dialer/
	  EOH
	end

	cookbook_file "/usr/share/elasticsearch/config_dialer/jvm.options" do
	  source "jvm.options"
	  owner   "elasticsearch"
	  group   "elasticsearch"
	  mode    "0644"
	end
	
	if node.inconcert.elasticsearch_dialer_nodes.size == 1 then
		template "/usr/share/elasticsearch/config_dialer/elasticsearch.yml" do 
		  source "elasticsearch.yml-single_node_dialer.erb"
		  owner  "elasticsearch"
		  group  "elasticsearch"
		  mode   "0644"
		end
	else
		template "/usr/share/elasticsearch/config_dialer/elasticsearch.yml" do 
		  source "elasticsearch.yml-cluster_dialer.erb"
		  owner  "elasticsearch"
		  group  "elasticsearch"
		  mode   "0644"
		end
	end
	
	node.elasticsearch_dialer.es_path_to_data.each do |d| 
	  directory d do
		recursive true
		owner  "elasticsearch"
		group  "elasticsearch"
		mode   "0700"
		action :create
	  end
	end

	directory "#{node.elasticsearch_dialer.es_path_to_log}" do
	  recursive true
	  owner  "elasticsearch"
	  group  "elasticsearch"
	  mode   "0700"
	  action :create
	end
	
	node.elasticsearch_dialer.es_path_to_data.each do |d| 
		bash "[ELASTIC-SEARCH-DIALER] permission for data directory" do
			cwd "#{d}"
			code <<-EOC
				chown -R elasticsearch:elasticsearch #{d}
				chmod -R 775  #{d}
			EOC
		end
	end
	
	bash "[ELASTIC-SEARCH-DIALER] permission for log directory" do
	  cwd "#{node.elasticsearch_dialer.es_path_to_log}"
	  code <<-EOC
		chown -R elasticsearch:elasticsearch #{node.elasticsearch_dialer.es_path_to_log}
		chmod -R 775  #{node.elasticsearch_dialer.es_path_to_log}
	  EOC
	end
	
	dir_service="/etc/inconcert/sv/elasticsearch_dialer"
	if not Dir.exist?(dir_service) then
		bash "[ELASTIC-SEARCH-DIALER] Create Elasticsearch service directory" do
		  user    "root"
		  cwd     "/root"
		  code  <<-EOH
			mkdir -p /etc/inconcert/sv/elasticsearch_dialer
		  EOH
		end
	end
	
	cookbook_file "/etc/inconcert/sv/elasticsearch_dialer/run" do
	  source "run_dialer"
	  owner   "root"
	  group   "root"
	  mode    "0777"
	end
	
	bash "[ELASTIC-SEARCH-DIALER] Create Elasticsearch Dialer service" do
	  user    "root"
	  cwd     "/root"
	  code  <<-EOH
		ln -s /etc/inconcert/sv/elasticsearch_dialer /etc/service/elasticsearch_dialer
		/usr/sbin/update-service --add /etc/inconcert/sv/elasticsearch_dialer
	  EOH
	end

	bash "[ELASTIC-SEARCH-DIALER] Set permission to user elasticsearch" do
	  cwd "/usr/share/elasticsearch"
	  code <<-EOC
		chown -R elasticsearch:elasticsearch /usr/share/elasticsearch
		chmod -R 775  /usr/share/elasticsearch
	  EOC
	end
	
	#BACKUPS De ES ------------------------------------------------------------
	if node.inconcert.key?("elasticsearch_dialer_back_up") && node.inconcert.elasticsearch_dialer_back_up.size > 0 && node.inconcert.elasticsearch_dialer_back_up['enable'] then

		if !node.inconcert.elasticsearch_dialer_back_up['path'].is_a? String then
			node.inconcert.elasticsearch_dialer_back_up['path'] = '/elasticsearch/recipe_bakcup'
		end

		if  node.inconcert.elasticsearch_dialer_back_up['path'] == "" then
			node.inconcert.elasticsearch_dialer_back_up['path'] = '/elasticsearch/recipe_bakcup'
		end

		privatePath = node.inconcert.elasticsearch_dialer_back_up['path'].reverse.sub("/".reverse, "/.".reverse).reverse

		if node['ipaddress'] == node.inconcert.elasticsearch_dialer_ip then
			
			package "nfs-kernel-server"

			bash "[ELASTIC-SEARCH-DIALER-BACKUP - Create NFS directory]" do
				user    "root"
				cwd     "/root"
				code  <<-EOH
					mkdir #{node.inconcert.elasticsearch_dialer_back_up['path']} -p
					mkdir #{privatePath} -p
					chown elasticsearch:elasticsearch #{node.inconcert.elasticsearch_dialer_back_up['path']}
					chown elasticsearch:elasticsearch #{privatePath}

					f=#{node.inconcert.elasticsearch_dialer_back_up['path']}
					while [[ $f != / ]]; do 
						chmod o+rx "$f"; 
						f=$(dirname "$f"); 
						echo "apply privileges to $f"; 
					done;
				EOH
			end
			bash "[ELASTIC-SEARCH-DIALER-BACKUP - Add directorty to NFS config file]" do
				user    "root"
				cwd     "/root"
				code  <<-EOH
					echo "#{privatePath}   *(rw,sync,no_root_squash,no_subtree_check) " >>  /etc/exports
					echo "Directoy NFS for ES backup add to configuration file"
				EOH
				not_if "grep -wq \"#{privatePath}\" /etc/exports"
			end
			bash "[ELASTIC-SEARCH-DIALER-BACKUP - Active NFS directory]" do
				user    "root"
				cwd     "/root"
				code  <<-EOH
					systemctl restart nfs-kernel-server
					exportfs -rav
				EOH
			end
		else
			package "nfs-common"
			bash "[ELASTIC-SEARCH-DIALER-BACKUP - Create backup directory for backups]" do
				user    "root"
				cwd     "/root"
				code  <<-EOH
					mkdir -p #{node.inconcert.elasticsearch_dialer_back_up['path']}
					f=#{node.inconcert.elasticsearch_dialer_back_up['path']}
					while [[ $f != / ]]; 
						do 
						chmod o+rx "$f";
						f=$(dirname "$f"); 
						echo "apply privileges to $f"; done;
				EOH
			end
		end

		bash "[ELASTIC-SEARCH-DIALER-BACKUP - Connect to NFS]" do
			user    "root"
			cwd     "/root"
			code  <<-EOH
				mount #{node.inconcert.elasticsearch_dialer_ip}:#{privatePath} #{node.inconcert.elasticsearch_dialer_back_up['path']}
			EOH
		end

		bash "[ELASTIC-SEARCH-DIALER-BACKUP - Add backup directorty to partition table]" do
			user    "root"
			cwd     "/root"
			code  <<-EOH
				echo "#{node.inconcert.elasticsearch_dialer_ip}:#{privatePath}  #{node.inconcert.elasticsearch_dialer_back_up['path']}  nfs auto,nofail,noatime,nolock,intr,tcp,actimeo=1800 0 0" >> /etc/fstab
				echo "Directoy NFS for ES backup add to partition file"
			EOH
			not_if "grep -wq \"#{privatePath}\" /etc/fstab"
		end

		bash "[ELASTIC-SEARCH-DIALER-BACKUP - Add path.repo to ES config file]" do
			user    "root"
			cwd     "/root"
			code  <<-EOH
				echo "add #{node.inconcert.elasticsearch_dialer_back_up['path']} to config file (elasticsearch.yml)"
				echo "" >> /usr/share/elasticsearch/config_dialer/elasticsearch.yml
				echo "path.repo: #{node.inconcert.elasticsearch_dialer_back_up['path']}," >> /usr/share/elasticsearch/config_dialer/elasticsearch.yml
				sv stop elasticsearch_dialer
				sv start elasticsearch_dialer
			EOH
			not_if 'grep -wq "path.repo" /usr/share/elasticsearch/config_dialer/elasticsearch.yml'
		end

		if node['ipaddress'] != node.inconcert.elasticsearch_dialer_ip || node.inconcert.elasticsearch_dialer_nodes.size == 1 then
			bash "[ELASTIC-SEARCH-DIALER-BACKUP - Config ES snapshot]" do
				user    "root"
				cwd     "/root"
				code  <<-EOH
					sleep 20
					retention_days=#{node.inconcert.elasticsearch_dialer_back_up['retention_days']}
					if [ -z "$retention_days" ]; then
						retention_days="30"
					fi

					curl -X PUT "localhost:#{node.inconcert.elasticsearch_dialer_port}/_snapshot/es_dialer?pretty" -H 'Content-Type: application/json' -d'
					{
						"type": "fs",
						"settings": {
							"location": "#{node.inconcert.elasticsearch_dialer_back_up['path']}"
						}
					}'

					curl -X PUT "localhost:#{node.inconcert.elasticsearch_dialer_port}/_slm/policy/nightly-snapshots?pretty" -H 'Content-Type: application/json' -d'
					{
						"schedule": "0 30 1 * * ?",       
						"name": "<nightly-snap-{now/d}>", 
						"repository": "es_dialer",    
						"config": {
							"indices": "*,-.*",                 
							"include_global_state": false    
						},
						"retention": {                    
							"expire_after": "'$retention_days'd",
							"min_count": 5,
							"max_count": 50
						}
					}'
				EOH
			end
		end
		bash "[ELASTIC-SEARCH-DIALER-BACKUP - Info ]" do
			user    "root"
			cwd     "/root"
			code  <<-EOH
				echo "======================================================================="
				echo "                   Elasticsearch backup enabled!"
				echo "======================================================================="
			EOH
		end
	else
		bash "[ELASTIC-SEARCH-DIALER-BACKUP] - Info " do
			user    "root"
			cwd     "/root"
			code  <<-EOH
				echo "========================================================="
				echo "         Elasticsearch backups not enabled"
				echo "========================================================="
			EOH
		end
	end
end
	
bash "[ELASTIC-SEARCH] Configure inConcert SHARDS and REPLICAS" do
  user "root"
  code <<-EOC
	set -e
	sed -i -e  "s/NUMBER_OF_SHARDS=.*/NUMBER_OF_SHARDS=#{node.elasticsearch.number_of_shards}/" /etc/inconcert/inconcert.conf
	sed -i -e  "s/NUMBER_OF_REPLICAS=.*/NUMBER_OF_REPLICAS=#{node.elasticsearch.number_of_replicas}/" /etc/inconcert/inconcert.conf
  EOC
  only_if "ls /etc/inconcert/inconcert.conf"
end

bash "[ELASTIC-SEARCH] Configure ES port" do
  user "root"
  code <<-EOC
	set -e
	sed -i -e  "s/ELASTICSEARCH_PORT=.*/ELASTICSEARCH_PORT=#{node.inconcert.elasticsearch_port}/" /etc/inconcert/inconcert.conf
  EOC
  only_if "ls /etc/inconcert/inconcert.conf"
end


bash "[ELASTIC-SEARCH-DIALER] Configure inConcert SHARDS and REPLICAS" do
  user "root"
  code <<-EOC
	set -e
	sed -i -e  "s/NUMBER_OF_SHARDS_DIALER=.*/NUMBER_OF_SHARDS_DIALER=#{node.elasticsearch_dialer.number_of_shards}/" /etc/inconcert/inconcert.conf
	sed -i -e  "s/NUMBER_OF_REPLICAS_DIALER=.*/NUMBER_OF_REPLICAS_DIALER=#{node.elasticsearch_dialer.number_of_replicas}/" /etc/inconcert/inconcert.conf
  EOC
  only_if "ls /etc/inconcert/inconcert.conf"
end
bash "[ELASTIC-SEARCH-DIALER] Configure ES Dialer port" do
  user "root"
  code <<-EOC
	set -e
	sed -i -e  "s/ELASTICSEARCH_DIALER_PORT=.*/ELASTICSEARCH_DIALER_PORT=#{node.inconcert.elasticsearch_dialer_port}/" /etc/inconcert/inconcert.conf
  EOC
  only_if "ls /etc/inconcert/inconcert.conf"
end

if esNode or esDialerNode then
	bash "[ELASTIC-SEARCH] Tocuh 'init elasticSearch'" do
	  cwd "/usr/share/elasticsearch"
	  code <<-EOC
		touch .bootstrapped_es-#{node.elasticsearch.version}
	  EOC
	end
end
