#
# Cookbook Name:: grafana
# Recipe:: default
#
# Copyright 2012, inConcert
#
# All rights reserved - Do Not Redistribute
#


#
# make helper methods available inside templates.
#
#class Erubis::Context
#	include Redis::Helpers
#end

Encoding.default_external = Encoding::UTF_8
Encoding.default_internal = Encoding::UTF_8
package "fonts-dejavu-core"
package "fontconfig-config"
package "libfontconfig1"
#Instalo Grafana ====================================================
bash "[GRAFANA] Download grafana_#{node.grafana.version_grafana}_amd64.deb" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	wget -q http://#{node.grafana.sources.server}/repo/grafana_#{node.grafana.version_grafana}_amd64.deb -O /tmp/grafana_#{node.grafana.version_grafana}_amd64.deb
  EOH
  not_if "ls /var/lib/grafana/.bootstrapped_grafana-#{node.grafana.version_grafana}"
end

bash "[GRAFANA] Install grafana_#{node.grafana.version_grafana}" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	dpkg -i grafana_#{node.grafana.version_grafana}_amd64.deb
  EOH
  not_if "ls /var/lib/grafana/.bootstrapped_grafana-#{node.grafana.version_grafana}"
end

bash "[GRAFANA] Install plugin redis" do
  user    "root"
  code  <<-EOH
	grafana-cli plugins install redis-datasource
  EOH
end

bash "[GRAFANA] Install plugin digiapulssi-breadcrumb-panel" do
  user    "root"
  code  <<-EOH
    grafana-cli plugins install digiapulssi-breadcrumb-panel
  EOH
end

bash "[GRAFANA] Install plugin grafana-polystat-panel" do
  user    "root"
  code  <<-EOH
  grafana-cli plugins install grafana-polystat-panel
  EOH
end

bash "[GRAFANA] Install plugin grafana-image-renderer" do
  user    "root"
  code  <<-EOH
  grafana-cli plugins install grafana-image-renderer
  EOH
end

template "/etc/grafana/grafana.ini" do
  source "grafana.ini_alone.erb"
  owner   "root"
  group   "root"
  mode    "0644"
end

cookbook_file "/usr/share/grafana/conf/defaults.ini" do
  source "defaults.ini"
  owner   "root"
  group   "root"
  mode    "0644"
end

cookbook_file "/etc/grafana/provisioning/datasources/datasources.yaml" do
  source "datasources.yaml"
  owner   "root"
  group   "root"
  mode    "0644"
end



#mysql
if node.grafana.include?(:mysql_nodes) && node.grafana.mysql_nodes.size > 0
  cookbook_file "/etc/grafana/provisioning/dashboards/MySQLOverview.json" do
    source "MySQLOverview.json"
    owner   "root"
    group   "root"
    mode    "0644"
  end
  cookbook_file "/etc/grafana/provisioning/dashboards/MySQLInnoDBDetails.json" do
    source "MySQLInnoDBDetails.json"
    owner   "root"
    group   "root"
    mode    "0644"
  end
  cookbook_file "/etc/grafana/provisioning/dashboards/MySQLPerformanceSchema.json" do
    source "MySQLPerformanceSchema.json"
    owner   "root"
    group   "root"
    mode    "0644"
  end
end

template "/etc/grafana/provisioning/notifiers/default.yaml" do
  source "notifier_default.erb"
  owner   "root"
  group   "root"
  mode    "0644"
end


# redis
if node.grafana.include?(:redis_nodes_to_monitor) && node.grafana.redis_nodes_to_monitor.size > 0
  node.grafana.redis_nodes_to_monitor.map do |n|
    if n['id'].is_a? String then
      if n['redis_ip'].size > 0 then
        node.grafana.redis_ip_node = n['redis_ip']
      else
        raise "Debe ingresar la IP en el nodo n['id']"
      end
      if n['redis_port'].size > 0 then
        node.grafana.redis_port_node = n['redis_port']
      else
        raise "Debe ingresar el puerto en el nodo n['id']"
      end
      if n['redis_agentservice_port'].size > 0 then
        node.grafana.redis_agentservice_port_node = n['redis_agentservice_port']
      else
        raise "Debe ingresar el puerto en el nodo n['id']"

      end    

      cookbook_file "/etc/grafana/provisioning/datasources/redis-datasource-#{node.grafana.redis_ip_node}-#{node.grafana.redis_port_node}.yaml" do
        source "redis-datasource"
        owner   "root"
        group   "root"
        mode    "0777"
      end  

      cookbook_file "/etc/grafana/provisioning/datasources/redis-datasource-#{node.grafana.redis_ip_node}-#{node.grafana.redis_agentservice_port_node}.yaml" do
        source "redis-datasource"
        owner   "root"
        group   "root"
        mode    "0777"


      end    
      
      bash "Configure redis datasource" do
        user "root"
        code <<-EOC 
        set -e
        sed -i -e  "s/REDIS_IP/#{node.grafana.redis_ip_node}/" /etc/grafana/provisioning/datasources/redis-datasource-#{node.grafana.redis_ip_node}-#{node.grafana.redis_port_node}.yaml
        sed -i -e  "s/REDIS_PORT/#{node.grafana.redis_port_node}/" /etc/grafana/provisioning/datasources/redis-datasource-#{node.grafana.redis_ip_node}-#{node.grafana.redis_port_node}.yaml
        EOC
      end

      bash "Configure redis agentservice datasource" do
        user "root"
        code <<-EOC 
        set -e
        sed -i -e  "s/REDIS_IP/#{node.grafana.redis_ip_node}/" /etc/grafana/provisioning/datasources/redis-datasource-#{node.grafana.redis_ip_node}-#{node.grafana.redis_agentservice_port_node}.yaml
        sed -i -e  "s/REDIS_PORT/#{node.grafana.redis_agentservice_port_node}/" /etc/grafana/provisioning/datasources/redis-datasource-#{node.grafana.redis_ip_node}-#{node.grafana.redis_agentservice_port_node}.yaml
        EOC
      end  
    end
  end

  cookbook_file "/etc/grafana/provisioning/dashboards/redis-dashboard.json" do
    source "redis-dashboard.json"
    owner   "root"
    group   "root"
    mode    "0644"
  end
end

cookbook_file "/etc/grafana/provisioning/dashboards/dashboards.yml" do
  source "dashboards.yml"
  owner   "root"
  group   "root"
  mode    "0644"
end

if node.grafana.include?(:servers_to_monitor) && node.grafana.servers_to_monitor.size > 0 || node.grafana.include?(:cassandra_nodes_to_monitor) && node.grafana.cassandra_nodes_to_monitor.size > 0
  cookbook_file "/etc/grafana/provisioning/dashboards/ServerMonitorBasic.json" do
    source "ServerMonitorBasic.json"
    owner   "root"
    group   "root"
    mode    "0644"
  end

  cookbook_file "/etc/grafana/provisioning/dashboards/ServerMonitorFull.json" do

    source "ServerMonitorFull.json"
    owner   "root"
    group   "root"
    mode    "0644"
  end
end

if node.grafana.include?(:snmp_services) && node.grafana.snmp_services.size > 0
  cookbook_file "/etc/grafana/provisioning/dashboards/SNMPInconcertServices.json" do
    source "SNMPInconcertServices.json"
    owner   "root"
    group   "root"
    mode    "0644"
  end
end

bash "[GRAFANA] Reset initial user" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	/etc/init.d/grafana-server stop
	rm -f /var/lib/grafana/grafana.db
  EOH
  not_if "ls /var/lib/grafana/.bootstrapped_grafana-#{node.grafana.version_grafana}"
end

bash "[GRAFANA] Tocuh 'init GRAFANA'" do
  cwd "/var/lib/grafana"
  code <<-EOC
	touch .bootstrapped_grafana-#{node.grafana.version_grafana}
  EOC
end

#Instalo Prometheus ====================================================
bash "[PROMETHEUS] Download prometheus-#{node.grafana.version_prometheus}.linux-amd64.tar.gz" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	wget -q http://#{node.grafana.sources.server}/repo/prometheus-#{node.grafana.version_prometheus}.linux-amd64.tar.gz -O /tmp/prometheus-#{node.grafana.version_prometheus}.linux-amd64.tar.gz
  EOH
  not_if "ls /opt/prometheus/.bootstrapped_prometheus-#{node.grafana.version_prometheus}"
end

bash "[PROMETHEUS] Install prometheus-#{node.grafana.version_prometheus}-amd64" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	tar -xvzf prometheus-#{node.grafana.version_prometheus}.linux-amd64.tar.gz
	mkdir /opt/prometheus
	cp -r prometheus-#{node.grafana.version_prometheus}.linux-amd64/* /opt/prometheus/
	rm -f prometheus-#{node.grafana.version_prometheus}-amd64.tar.gz
	rm -rf prometheus-#{node.grafana.version_prometheus}.linux-amd64
  EOH
  not_if "ls /opt/prometheus/.bootstrapped_prometheus-#{node.grafana.version_prometheus}"
end

cookbook_file "/etc/init.d/prometheus" do
  source "prometheus"
  owner   "root"
  group   "root"
  mode    "0777"
end


bash "[PROMETHEUS] Tocuh 'init PROMETHEUS'" do
  cwd "/opt/prometheus"
  code <<-EOC
	touch .bootstrapped_prometheus-#{node.grafana.version_prometheus}
  EOC
end

#Instalo elasticsearch_exporter ====================================================
bash "[ELASTICSEARCH_EXPLORER] Download elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}.linux-amd64.tar.gz" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	wget -q http://#{node.grafana.sources.server}/repo/elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}.linux-amd64.tar.gz -O /tmp/elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}.linux-amd64.tar.gz
  EOH
  not_if "ls /opt/elasticsearch_exporter/.bootstrapped_elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}"
end

bash "[ELASTICSEARCH_EXPLORER] Install elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}.linux-amd64" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	tar -xvzf elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}.linux-amd64.tar.gz
	mkdir /opt/elasticsearch_exporter
	cp -r elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}.linux-amd64/* /opt/elasticsearch_exporter/
	rm -f elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}.linux-amd64.tar.gz
	rm -rf elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}.linux-amd64
  EOH
  not_if "ls /opt/elasticsearch_exporter/.bootstrapped_elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}"
end

bash "[ELASTICSEARCH_EXPLORER] Tocuh 'init ELASTICSEARCH_EXPLORER'" do
  cwd "/opt/elasticsearch_exporter"
  code <<-EOC
	touch .bootstrapped_elasticsearch_exporter-#{node.grafana.version_elasticsearch_exporter}
  EOC
end

#Configuro instancias a Monitorear ====================================================
template "/opt/prometheus/prometheus.yml" do
  source "prometheus.yml_alone.erb"
  owner   "root"
  group   "root"
  mode    "0644"
end

bash "[PROMETHEUS] validate promethus config" do
  cwd "/"
  code <<-EOC
	  /opt/prometheus/promtool check config /opt/prometheus/prometheus.yml
  EOC
end


if node.grafana.include?(:elasticsearch_nodes_to_monitor) && node.grafana.elasticsearch_nodes_to_monitor.size > 0
  cookbook_file "/etc/grafana/provisioning/dashboards/elasticsearch_rev4.json" do
    source "elasticsearch_rev4.json"
    owner   "root"
    group   "root"
    mode    "0644"
  end  
#killall -9 -q elasticsearch_exporter
  bash "[ELASTICSEARCH_EXPLORER] Delete old configuration" do
    user    "root"
    cwd     "/tmp"
    code  <<-EOH
    rm -f /etc/init.d/elasticsearch_exporter_*
    EOH

  end


  node.grafana.explorer_port=9053
  node.grafana.elasticsearch_nodes_to_monitor.map do |n|
    if n['id'].is_a? String then
      if n['elasticsearch_ip'].size > 0 then
        node.grafana.elasticsearch_ip_node = n['elasticsearch_ip']
      else
        raise "Debe ingresar la IP en el nodo n['id']"
      end
      if n['elasticsearch_port'].size > 0 then
        node.grafana.elasticsearch_port_node = n['elasticsearch_port']
      else
        raise "Debe ingresar el puerto en el nodo n['id']"
      end
      cookbook_file "/etc/init.d/elasticsearch_exporter_#{node.grafana.explorer_port}" do
        source "elasticsearch_exporter"
        owner   "root"
        group   "root"
        mode    "0777"
      end

      
      bash "Configure elasticsearch_exporter init" do
        user "root"
        code <<-EOC 
        set -e
        sed -i -e  "s/NAME=.*/NAME=elasticsearch_exporter_#{node.grafana.explorer_port}/" /etc/init.d/elasticsearch_exporter_#{node.grafana.explorer_port}
        sed -i -e  "s/ELASTICSEARCH_IP/#{node.grafana.elasticsearch_ip_node}/" /etc/init.d/elasticsearch_exporter_#{node.grafana.explorer_port}
        sed -i -e  "s/ELASTICSEARCH_PORT/#{node.grafana.elasticsearch_port_node}/" /etc/init.d/elasticsearch_exporter_#{node.grafana.explorer_port}
        sed -i -e  "s/EXPLORER_PORT/#{node.grafana.explorer_port}/" /etc/init.d/elasticsearch_exporter_#{node.grafana.explorer_port}
        EOC
      end

      
      bash "[PROMETHEUS] Set Jobs" do
        user    "root"
        cwd     "/tmp"
        code  <<-EOH
        echo "  - job_name: elasticsearch_#{node.grafana.explorer_port}" >> /opt/prometheus/prometheus.yml
        echo "    scrape_interval: 60s" >> /opt/prometheus/prometheus.yml
        echo "    scrape_timeout:  30s" >> /opt/prometheus/prometheus.yml
        echo "    metrics_path: \"/metrics\"" >> /opt/prometheus/prometheus.yml
        echo "    static_configs:" >> /opt/prometheus/prometheus.yml
        echo "    - targets: ['0.0.0.0:#{node.grafana.explorer_port}' ]" >> /opt/prometheus/prometheus.yml
        echo "      labels:" >> /opt/prometheus/prometheus.yml
        echo "        service: elasticsearch" >> /opt/prometheus/prometheus.yml
        echo "" >> /opt/prometheus/prometheus.yml
        EOH
      end

      node.grafana.explorer_port=node.grafana.explorer_port + 1
    end
  end
end

#Restart services ====================================================
bash "[GRAFANA] Restart GRAFANA" do
  cwd "/etc/init.d"
  code <<-EOC
	/etc/init.d/grafana-server restart
  EOC
end

if node.grafana.include?(:elasticsearch_nodes_to_monitor) && node.grafana.elasticsearch_nodes_to_monitor.size > 0
  node.grafana.explorer_port=9053
  node.grafana.elasticsearch_nodes_to_monitor.map do |n|
    if n['id'].is_a? String then

      
      bash "[ELASTICSEARCH_EXPLORER] Restart ELASTICSEARCH_EXPLORER elasticsearch_exporter_#{node.grafana.explorer_port}" do
        cwd "/etc/init.d"
        code <<-EOC
        /etc/init.d/elasticsearch_exporter_#{node.grafana.explorer_port} restart
        EOC
      end

      
      node.grafana.explorer_port=node.grafana.explorer_port + 1
    end
  end
end

bash "[PROMETHEUS] Restart PROMETHEUS" do
  cwd "/etc/init.d"
  code <<-EOC
	/etc/init.d/prometheus restart
  EOC
end

bash "[GRAFANA] Restart GRAFANA" do
  cwd "/etc/init.d"
  code <<-EOC
	/etc/init.d/grafana-server restart
  EOC
end

#Instalo node exporter ====================================================
bash "[NODE_EXPORTER] Download https://github.com/prometheus/node_exporter/releases/download/v1.3.1/node_exporter-1.3.1.linux-amd64.tar.gz" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    wget -q https://github.com/prometheus/node_exporter/releases/download/v1.3.1/node_exporter-1.3.1.linux-amd64.tar.gz -O /tmp/node_exporter-1.3.1.linux-amd64.tar.gz
  EOH
  not_if "ls /etc/prometheus_exporters/node_exporter/node_exporter"
end

bash "[NODE_EXPORTER] Install node_exporter-1.3.1.linux-amd64.tar.gz" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    tar -xvzf node_exporter-1.3.1.linux-amd64.tar.gz
    mkdir -p /etc/prometheus_exporters/node_exporter
    cp -r node_exporter-1.3.1.linux-amd64/* /etc/prometheus_exporters/node_exporter/
    rm -f node_exporter-1.3.1.linux-amd64.tar.gz
    rm -rf node_exporter-1.3.1.linux-amd64
  EOH
  not_if "ls /etc/prometheus_exporters/node_exporter/node_exporter"
end

cookbook_file "/etc/systemd/system/node_exporter.service" do
  source "node_exporter.service"
  owner   "root"
  group   "root"
  mode    "0644"
end

bash "[NODE_EXPORTER] add to system service" do
  user    "root"
  cwd "/"
  code <<-EOC
    sudo systemctl daemon-reload
    sudo systemctl enable node_exporter
    sudo systemctl start node_exporter  
  EOC
end


#Instalo pushgateway ====================================================
bash "[PUSHGATEWAY] Download https://github.com/prometheus/pushgateway/releases/download/v1.4.2/pushgateway-1.4.2.linux-amd64.tar.gz" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    wget -q https://github.com/prometheus/pushgateway/releases/download/v1.4.2/pushgateway-1.4.2.linux-amd64.tar.gz -O /tmp/pushgateway-1.4.2.linux-amd64.tar.gz
  EOH
  not_if "ls /etc/prometheus_exporters/pushgateway/pushgateway"
end

bash "[PUSHGATEWAY] Install pushgateway-1.4.2.linux-amd64.tar.gz" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    tar -xvzf pushgateway-1.4.2.linux-amd64.tar.gz
    mkdir -p /etc/prometheus_exporters/pushgateway
    cp -r pushgateway-1.4.2.linux-amd64/* /etc/prometheus_exporters/pushgateway
    rm -f pushgateway-1.4.2.linux-amd64.tar.gz
    rm -rf pushgateway-1.4.2.linux-amd64
  EOH
  not_if "ls /etc/prometheus_exporters/node_exporter/pushgateway"
end

cookbook_file "/etc/systemd/system/pushgateway.service" do
  source "pushgateway.service"
  owner   "root"
  group   "root"
  mode    "0644"
end

bash "[PUSHGATEWAY] add to system service" do
  user    "root"
  cwd "/"
  code <<-EOC
    sudo systemctl daemon-reload
    sudo systemctl enable pushgateway
    sudo systemctl start pushgateway  
  EOC
end

cookbook_file "/etc/prometheus_exporters/pushgateway/process-top" do
  source "process-top"
  owner   "root"
  group   "root"
  mode    "0744"
end

cookbook_file "/etc/prometheus_exporters/pushgateway/push_top.sh" do
  source "push_top.sh"
  owner   "root"
  group   "root"
  mode    "0744"
end

cookbook_file "/etc/systemd/system/pushtop.service" do
  source "pushtop.service"
  owner   "root"
  group   "root"
  mode    "0644"
end

bash "[PUSHGATEWAY] add to push top" do
  user    "root"
  cwd "/"
  code <<-EOC
    systemctl daemon-reload
    systemctl enable pushtop
    systemctl start pushtop
  EOC
end