#
# Cookbook Name:: HotFixer
# Recipe:: default
#
# Copyright 2022, inConcert
#
# All rights reserved - Do Not Redistribute
#
#!/bin/env ruby
# encoding: utf-8


bash "[HOTFIXER] Valid host " do
	user    "root"
	cwd     "/"
	code  <<-EOH
	    response=$(curl -m 1 -o /dev/null -u myself:XXXXXX -Isw '%{http_code}\n' http://#{node.hotfixer.sources.server})
		if [[ "$response" == "000" ]]; then
			echo =========================================================================================================================================
			echo "                        Error connecting to #{node.hotfixer.sources.server} !"
			echo  =========================================================================================================================================
			exit "Error"
		fi
	EOH
end



if node[:platform].include?("debian")


	bash "[HOTFIXER] hotfixer version " do
		user    "root"
		cwd     "/"
		code  <<-EOH
			echo =========================================================================================================================================
			echo "                                               Debian detected!"
			echo =========================================================================================================================================
		EOH
	end

	bash "[HOTFIXER] Install jq" do
		user    "root"
		cwd     "/"
		code  <<-EOH
		wget -q http://#{node.hotfixer.sources.server}/repo/libonig4_6.1.3-2+deb9u2_amd64.deb -O /tmp/libonig4_6.1.3-2+deb9u2_amd64.deb
		wget -q http://#{node.hotfixer.sources.server}/repo/libonig5_6.9.1-1_amd64.deb -O /tmp/libonig5_6.9.1-1_amd64.deb
		wget -q http://#{node.hotfixer.sources.server}/repo/libjq1_1.5+dfsg-2+b1_amd64.deb -O /tmp/libjq1_1.5+dfsg-2+b1_amd64.deb
		wget -q http://#{node.hotfixer.sources.server}/repo/jq_1.5+dfsg-2+b1_amd64.deb -O /tmp/jq_1.5+dfsg-2+b1_amd64.deb
		EOH
	end

	bash "[HOTFIXER] Install jq" do
		user    "root"
		cwd     "/tmp"
		code  <<-EOH
			dpkg -i libonig4_6.1.3-2+deb9u2_amd64.deb
			dpkg -i libonig5_6.9.1-1_amd64.deb
			dpkg -i libjq1_1.5+dfsg-2+b1_amd64.deb
			dpkg -i jq_1.5+dfsg-2+b1_amd64.deb
		EOH
	end

	bash "[HOTFIXER] jq test" do
		user    "root"
		cwd     "/"
		code  <<-EOH
			echo "Command jq  version"
			jq --version
		EOH
	end
else
	package "jq"
end

cookbook_file "/getOS.sh" do
	source "getOS.sh"
	owner   "root"
	group   "root"
	mode    "0744"
 end

cookbook_file "/getInconertVersion.sh" do
	source "getInconertVersion.sh"
	owner   "root"
	group   "root"
	mode    "0744"
 end
 
cookbook_file "/hotfixerversion.txt" do
	source  "hotfixerversion.txt"
	owner   "root"
	group   "root"
	mode    "0644"
end

bash "[HOTFIXER] hotfixer version" do
	user    "root"
	cwd     "/"
	code  <<-EOH
		sed -i 's/#version#/#{node.hotfixer.version}/g' /hotfixerversion.txt
	EOH
end

# valid dependences
cookbook_file "/cassandra_connection.lua" do
	source "cassandra_connection.lua"
	owner   "root"
	group   "root"
	mode    "0744"
end
  
cookbook_file "/checkCassandraConnection.sh" do
	source "checkCassandraConnection.sh"
	owner   "root"
	group   "root"
	mode    "0744"
 end
  
cookbook_file "/getHotfixesInstalled.sh" do
	source "getHotfixesInstalled.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/GetHotfixesInstalados.lua" do
	source "GetHotfixesInstalados.lua"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/fixInstalledChecked.txt" do
	source "fixInstalledChecked.txt"
	owner   "root"
	group   "root"
	mode    "0744"
end
	
cookbook_file "/extractDependences.sh" do
	source "extractDependences.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/validateInconcertDependency.sh" do
	source "validateInconcertDependency.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/processDependenciesStatus.sh" do
	source "processDependenciesStatus.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end
# end valid dependences

#install hotfixes

cookbook_file "/hotfixerBackup.sh" do
	source "hotfixerBackup.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/execBashHotfixFiles.sh" do
	source "execBashHotfixFiles.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/20220602T235959_XXXX_CreateColumnFamily.sh" do
	source "20220602T235959_XXXX_CreateColumnFamily.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/execute_insert_hotfix.sh" do
	source "execute_insert_hotfix.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/insertDatos_hotfix.lua" do
	source "insertDatos_hotfix.lua"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/UpdateHotfixID.lua" do
	source "UpdateHotfixID.lua"
	owner   "root"
	group   "root"
	mode    "0744"
end


cookbook_file "/updateHotfixesInstalled.sh" do
	source "updateHotfixesInstalled.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end

cookbook_file "/execute_update_hotfix.sh" do
	source "execute_update_hotfix.sh"
	owner   "root"
	group   "root"
	mode    "0744"
end


bash "[HOTFIXER] hotfixer version " do
	user    "root"
	cwd     "/"
	code  <<-EOH
		echo =========================================================================================================================================
		echo "                                               HOTFIXER v #{node.hotfixer.version}"
		echo =========================================================================================================================================
	EOH
end

bash "[HOTFIXER] Show inconcertcc version" do
	user    "root"
	cwd     "/"
	code  <<-EOH
        echo "HotFixer Working! ..."
        echo "Inconcert version "
		bash /getInconertVersion.sh
	EOH
end

cookbook_file "/checkVersion.sh" do
	source  "checkVersion.sh"
	owner   "root"
	group   "root"
	mode    "0644"
end

if node.hotfixer.key?("installAll") && node.hotfixer.installAll
	cookbook_file "/installAllHotfixes.sh" do
		source "installAllHotfixes.sh"
		owner   "root"
		group   "root"
		mode    "0744"
	end

	bash "[HOTFIXER] Install all hotfix" do
		user    "root"
		cwd     "/"
		code  <<-EOH
			inconcertv=$(bash /getInconertVersion.sh)
			echo "========================================================================================================================================="
			echo "                       Install all hotfix for Inconertcc version $inconcertv"
			echo "========================================================================================================================================="
			os=$(bash /getOS.sh)
			wget -q http://#{node.hotfixer.sources.server}/repo/hotfixes/"$inconcertv"_"$os"/allHotfix.txt -O /tmp/hotfixes/"$inconcertv"_"$os"/allHotfix.txt
			cat /tmp/hotfixes/"$inconcertv"_"$os"/allHotfix.txt
			echo
			echo "========================================================================================================================================="
			echo
			cp /tmp/hotfixes/"$inconcertv"_"$os"/allHotfix.txt /
			bash /installAllHotfixes.sh
		EOH
	end
	
	bash "Clean File System" do
		user    "root"
		cwd     "/"
		code  <<-EOH 
			rm /installAllHotfixes.sh
		EOH
		ignore_failure true
	end
else


	cookbook_file "/installOneToOneHotfix.sh" do
		source "installOneToOneHotfix.sh"
		owner   "root"
		group   "root"
		mode    "0744"
	end
	
	bash "[HOTFIXER] Install hotfixes" do
		user    "root"
		cwd     "/"
		code  <<-EOH
			inconcertv=$(bash /getInconertVersion.sh)
			echo "========================================================================================================================================="
			echo "                       Install config hotfix for Inconertcc version $inconcertv"
			echo "========================================================================================================================================="
			echo
			bash /installOneToOneHotfix.sh
		EOH
	end
	bash "Clean File System" do
		user    "root"
		cwd     "/"
		code  <<-EOH 
			rm /installOneToOneHotfix.sh
		EOH
		ignore_failure true
	end
end
