#!/bin/bash

#
# source cluster configuration
. /etc/inconcert/inconcert.conf
. /etc/inconcert/network.conf

#
# Adds, removes and updates variables on config files
function ast_set_variable()
{
	local conf_file="$1";
	local section="$2"
	local var_name="$(echo $3| sed -e 's/\//\\\//g' -e 's/\&/\\\&/g')";
	local var_value="$(echo $4 | sed -e 's/\//\\\//g' -e 's/\&/\\\&/g')";

	echo "[$conf_file] Setting $section/$var_name => $var_value"

	if [ -z "$(sed -n "/\[$section\]/,/^\[$/{ /$var_name/p }" $conf_file)" ]
	then # Variable does not exist
		if [ -z "$(grep "^\[$section\]$" $conf_file )" ]
		then # Section does not exist"
			echo -e "\n[$section]\n" >> $conf_file
		fi

		[ -z "$var_value" ] && var_name=";$var_name";
		sed -i "/\[$section\]/ a $var_name=>$var_value" $conf_file
	else
		if [ -z "$var_value" ]; then
			sed -i "/\[$section\]/,/^\[.*$/{s/^[;#]\{0,1\} *\<$var_name\> *=\([>]\{0,1\}\)\(.*\)/;$var_name=\1\2/}" "$conf_file";
		else # note: sets only the first occurrence of the variable
			sed -i "/\[$section\]/,/^\[.*$/{0,/^[;#]\{0,1\} *\<$var_name\> *=\([>]\{0,1\}\)\(.*\)/s//$var_name=\1$var_value/}" "$conf_file";
		fi
	fi
}

cd /etc/asterisk

if [ -f /etc/asterisk/inconcert.conf ]; then
	ast_set_variable "inconcert.conf" "global"        "cluster"              "$CLUSTER_ID"
	ast_set_variable "inconcert.conf" "global"        "listening_ip"         "$LOCAL_IP"
	ast_set_variable "inconcert.conf" "global"        "listening_port"       "$ASTERISK_LISTENING_PORT"
	ast_set_variable "inconcert.conf" "configcontrol" "server"               "$CONFIG_IP"
	ast_set_variable "inconcert.conf" "configcontrol" "port"                 "$CONFIG_PORT"
	ast_set_variable "http.conf" "general" "bindaddr"   "$LOCAL_IP"
fi