 
include_recipe "apt"

#paquetes auxiliares 
package "tmux"

#Operating System Identification
ubuntuDistro="UbuntuBionic"
if node["platform_version"].start_with? "22" then
	ubuntuDistro="UbuntuJammy"
end

# netplan to networkinterface
if node["platform"] == "ubuntu" and node["platform_version"].start_with? "22" then
  package "ifupdown"
  package "net-tools"
  
  cookbook_file "/configNetwork.sh" do
    source  "configNetwork.sh"
    owner   "root"
    mode    "0744"
    not_if "ls /etc/network/network"
  end
  bash "Config Network" do
    user "root"
    code "bash /configNetwork.sh"
    not_if "ls /etc/network/network"
  end  
  bash "restart services" do
	user    "root"
	cwd     "/tmp"
	code  <<-EOH
    echo "" >/etc/network/network 
		echo "Ready network config!"
	EOH
  end
  
  package "netplan.io" do
    action :remove
  end
  
  bash "remove config network file" do
    user "root"
    code "rm /configNetwork.sh"
    only_if "ls /configNetwork.sh"
  end

end

#paquetes LDAP
if node["platform"] == "ubuntu" and node["platform_version"].start_with? "22" then
	package "libldap-2.5"
	package "libharfbuzz-icu0"
	package "libicu-le-hb-dev"
	package "libicu70"
	package "libhpdf-2.3.0"
	package "libpng12-0"
	package "libharu"
else
	package "libldap-2.4"
end
package "libldap2-dev"

if node["platform"] == "ubuntu" then
	# RASA VERSION
	rasaCorrectVersion = node.inconcert.x_rasa_version.to_i
	installRasa = false
	rasaVersion = `pip3 show rasa-x | grep Version`
	rasaVersion=rasaVersion.split(":")
	bash "check version of rasa-x (> #{rasaCorrectVersion})" do
	  cwd   "/etc"
	  code  "echo rasa-x Version: #{rasaVersion[1]}"
	end
	if rasaVersion[0] == "Version" and rasaVersion[1] != "" then
		rasaVersionSplitted = rasaVersion[1].split(".")
		rasaNumber = (rasaVersionSplitted[0].to_i * 1000) + (rasaVersionSplitted[1].to_i * 100) + rasaVersionSplitted[2].to_i
		if rasaNumber < rasaCorrectVersion then
			installRasa = true
		else
			bash "check version of rasa-x" do
			  cwd   "/etc"
			  code  "echo 'x-Rasa is Updated'"
			end
		end
	else
		installRasa = true
	end
	if installRasa then
		include_recipe "i6::rasa_lib"
	end
end

#libicu
if node["platform"] == "ubuntu" and  ubuntuDistro=="UbuntuBionic" then
	package "libicu60" do
	  version "60.2-3ubuntu3.2"
	end
	package "libgraphite2-3" do
	  version "1.3.11-2"
	end
	package "libharfbuzz0b" do
	  version "1.7.2-1ubuntu1"
	end
	package " libicu-le-hb0" do
	  version "1.0.3+git161113-4"
	end
	package "libiculx60" do
	  version "60.2-3ubuntu3.2"
	end
	package "icu-devtools" do
	  version "60.2-3ubuntu3.2"
	end
	package "libicu-le-hb-dev" do
	  version "1.0.3+git161113-4"
	end
end

#Los paquetes de ffmpeg los instalo solo para Debian 8
if node["platform"] == "debian" then
	if node["platform_version"].start_with? "6" then
		#No puedo instalar ffmpeg
	else
		include_recipe "i6::ffmpeg"
	end
else 
	if node["platform"] == "ubuntu" then
		include_recipe "i6::ffmpeg"	
	end
end

#Los paquetes de msodbcsql los instalo solo para Debian 8
if node["platform"] == "debian" then
	if node["platform_version"].start_with? "6" then
		#No puedo instalar msodbcsql
	else
		include_recipe "i6::msodbcsql"
		include_recipe "i6::mysqlodbc"
	end
else 
	if node["platform"] == "ubuntu" then
		include_recipe "i6::msodbcsql"
		include_recipe "i6::mysqlodbc"
	end
end

#Opencv para Ubuntu
if node["platform"] == "ubuntu" then
	if node["platform_version"].start_with? "22" then
		package "python3-opencv"
	else
		package "python-opencv"
	end
	package "libopencv-dev"
end

#sox mp3 codecs para Ubuntu
if node["platform"] == "ubuntu" then
	package "libsox-fmt-mp3"
end

template "/var/cache/local/preseeding/inconcert.seed" do
  source "inconcert.seed.erb"
  owner  "root"
  group  "root"
  mode   "0644"
end

#  move this to a new recipe
# FIXME create a helper to check inconcert version and notify only if it changed
#
execute "packages update check" do
  command "ls"
  notifies :run, resources(:execute => "apt-get update"), :immediately
end

# Install lenguage into O.S.
cookbook_file "/etc/locale.gen" do
  source  "locale.gen"
  owner   "root"
  mode    "0744"
end

if node["platform"] == "debian" then
	if node["platform_version"].start_with? "6" then
		#en squeeze puedo no tener dos2unix
	else
		package "dos2unix"
		bash "dos2unix lengauge" do
		  cwd   "/etc"
		  code  "dos2unix /etc/locale.gen"
		end
	end
else
	package "dos2unix"
	bash "dos2unix lengauge" do
	  cwd   "/etc"
	  code  "dos2unix /etc/locale.gen"
	end
end

bash "configure o.s. lengauge" do
  cwd   "/etc"
  code  "locale-gen"
end

#Para Ubuntu instalo Luarocks con el paquete propio para que no genere problemas.
if node["platform"] == "ubuntu" then
	package "luarocks" do
	  version "2.2.0-1+inconcert"
	end
end

#package node.inconcert.dependencies.java.to_s do
#  # force yes is a hack because I do not have a signed repo.
#  options '--force-yes -o Dpkg::Options::="--force-confold" -o Dpkg::Options::="--force-confmiss" '
#end

# Hack: Before icc-rocks 6.0.1710+MCW-1710, the rock 'reporter' must be manually removed before
# installing a new version of the package. Later packages properly remove this rock.
bash "remove reporter rock" do
  user "root"
  code "dpkg --compare-versions `dpkg-query --show --showformat '${Version}' icc-rocks` lt 6.0.1713+MCW-1713 && luarocks remove --force reporter || :"
  only_if do
    current = `dpkg-query --show --showformat '${Version}' icc-rocks`
    node.inconcert.packages.icc_rocks != current
  end
end

cookbook_file "/tmp/backup_config.sh" do
  source "backup_config.sh"
  owner "root"
  group "root"
  mode  0744
end

bash "[OCC] BackUp INCONCERT CLEANUP DATA config" do
  user "root"
  code <<-EOC
	/tmp/backup_config.sh
  EOC
  only_if "ls /etc/inconcert/inconcert.conf"
end
package "icc-rocks" do
  # force yes is a hack because I do not have a signed repo.
  options '--force-yes -o Dpkg::Options::="--force-confnew" '
  version node.inconcert.packages.icc_rocks if node.inconcert.packages.icc_rocks != "*"
end

package "inconcert" do
  # force yes is a hack because I do not have a signed repo.
  options '--force-yes -o Dpkg::Options::="--force-confnew" '
  version node.inconcert.packages.inconcert if node.inconcert.packages.inconcert != "*"
end

# Obtengo ejecutable de Duckling
if node["platform"] == "ubuntu" then
	if node["platform_version"].start_with? "18" then
		bash "download icc-duckling-exe" do
		  user    "root"
		  cwd     "/tmp"
		  code  <<-EOH
			wget -q http://#{node.inconcert.sources.server}/repo/icc-duckling.gz -O /tmp/icc-duckling.gz
		  EOH
		end
	end
	if node["platform_version"].start_with? "22" then
		bash "download icc-duckling-exe" do
		  user    "root"
		  cwd     "/tmp"
		  code  <<-EOH
			wget -q http://#{node.inconcert.sources.server}/repo/icc-duckling_ubuntu_jammy.gz -O /tmp/icc-duckling.gz
		  EOH
		end
	end
	bash "Install icc-duckling-exe from gz file" do
	  user    "root"
	  cwd     "/tmp"
	  code  <<-EOH
		tar xvzf icc-duckling.gz -C /usr/sbin/
		chmod +x /usr/sbin/icc-duckling-exe
		
	  EOH
	end

end
# Hack: We have subsumed the libcassandra package inside the inconcert package.
# We need to uninstall it.
bash "remove libcassandra package" do
  user "root"
  code "dpkg --compare-versions `dpkg-query --show --showformat '${Version}' inconcert` gt 6.0.3035+MCW-3035 && apt-get remove libcassandra -q -y || :"
  #only_if do
  #  current = `dpkg-query --show --showformat '${Version}' libcassandra`
  #  node.inconcert.packages.inconcert != current
  #end
end

bash "reconfigure inconcert" do
  action :nothing
  user "root"
  code "dpkg-reconfigure --frontend=noninteractive inconcert"
end

cookbook_file "/etc/default/snmpd" do
  source  "snmpd-default"
  owner   "root"
  mode    "0744"
end

bash "configure inconcert snmp" do
  cwd   "/usr/share/inconcert/snmp"
  code  "./bootstrap.sh"
end

bash "preseed inconcert" do
  user "root"
  code "debconf-set-selections /var/cache/local/preseeding/inconcert.seed"
  # FIXME check if apt-get update is necessary at this point. slows things down terribly.
  notifies :run, resources(:execute => "apt-get update"), :immediately
  notifies :run, "bash[reconfigure inconcert]", :immediately
end

cookbook_file "/tmp/restore_config.sh" do
  source "restore_config.sh"
  owner "root"
  group "root"
  mode  0744
end

bash "[OCC] Restore INCONCERT CLEANUP DATA config" do
  user "root"
  code <<-EOC
	/tmp/restore_config.sh
  EOC
  only_if "ls /etc/inconcert/inconcert_cleanup_data_config.conf"
end
bash "Configure inConcert CONSISTENCY LEVEL" do
  user "root"
  code <<-EOC
    set -e
	sed -i -e  "s/CASSANDRA_DEFAULT_WRITE_CONSISTENCY_LEVEL=.*/CASSANDRA_DEFAULT_WRITE_CONSISTENCY_LEVEL=#{node.inconcert.consistency_level}/" /etc/inconcert/inconcert.conf
	sed -i -e  "s/CASSANDRA_DEFAULT_READ_CONSISTENCY_LEVEL=.*/CASSANDRA_DEFAULT_READ_CONSISTENCY_LEVEL=#{node.inconcert.consistency_level}/" /etc/inconcert/inconcert.conf
   
  EOC
  only_if {node.inconcert.consistency_level}
end

bash "Configure exclude node for cassandra" do
  user "root"
  code <<-EOC
    set -e
	sed -i -e  "s/export CASSANDRA_EXCLUDED_NODES=.*/export CASSANDRA_EXCLUDED_NODES=#{node.inconcert.exclude_nodes_cassandra}/" /etc/inconcert/inconcert.conf
  EOC
  only_if {node.inconcert.exclude_nodes_cassandra}
end

cookbook_file "/tmp/update_asterisk_config.sh" do
  source "update_asterisk_config.sh"
  owner "root"
  group "root"
  mode  0744
end

bash "[ASTERISK] Update asterisk config" do
  user "root"
  code <<-EOC
	dos2unix /tmp/update_asterisk_config.sh
	/tmp/update_asterisk_config.sh
  EOC
end
template "/etc/logrotate.d/inconcert" do
  source  "inconcert.logrotate.erb"
  owner   "root"
  group   "root"
  mode    "0644"
end

template "/etc/logrotate.d/inconcert_cleanupdata" do
  source  "inconcert_cleanupdata.logrotate.erb"
  owner   "root"
  group   "root"
  mode    "0644"
end

# run email proxy mail cleaner hourly
cookbook_file "/etc/cron.hourly/emailproxy_mail_cleaner" do
  source "emailproxy_mail_cleaner"
  owner "root"
  group "root"
  mode  0744
end

# run cores cleaner hourly
cookbook_file "/etc/cron.hourly/cores_cleaner" do
  source "cores_cleaner"
  owner "root"
  group "root"
  mode  0744
end

template "/usr/lib/inconcert/configserver/serverconfig/inconcert/apps_config.lua" do
  source "configserver_apps_config.lua.erb"
  owner  "inconcert"
  mode   "0744"
  # FIXME restart configserver
end

# seteo DefaultTasksMax para el caso de Ubuntu
if node["platform"] == "ubuntu" then
	bash "systemctl DefaultTasksMax " do 
	  user "root"
	  code <<-EOC
		sed -i -e  "s/#DefaultTasksMax=.*/DefaultTasksMax=infinity/" /etc/systemd/system.conf
		sed -i -e  "s/#DefaultTasksAccounting=yes.*/DefaultTasksAccounting=yes/" /etc/systemd/system.conf
		systemctl daemon-reload
	  EOC
	end
end

#actualizo la version de curl para soportar sftp
if node["platform"] == "ubuntu" then
	bash "download curl_7_73" do
	  user    "root"
	  cwd     "/tmp"
	  code  <<-EOH
		wget -q http://#{node.inconcert.sources.server}/repo/curl_7_73_ubuntu.gz -O /tmp/curl_7_73_ubuntu.gz
	  EOH
	end

	bash "Install curl_7_73 from gz file" do
	  user    "root"
	  cwd     "/tmp"
	  code  <<-EOH
		tar xvzf curl_7_73_ubuntu.gz
		cp /usr/bin/curl /usr/bin/curl_old
		cd curl_7_73_ubuntu
		cp lib/* /lib/
		ln -s /lib/libssh2.so.1.0.1 /lib/libssh2.so 
		ln -s /lib/libssh2.so.1.0.1 /lib/libssh2.so.1

		cp usr-bin/* /usr/bin/

		cp usr-lib/* /usr/lib/
		ln -s /usr/lib/libcurl.so.4.7.0 /usr/lib/libcurl.so
		ln -s /usr/lib/libcurl.so.4.7.0 /usr/lib/libcurl.so.4

		cp usr-include/* /usr/include/
		ln -s /usr/lib/libcurl.so.4 /lib/x86_64-linux-gnu/

		ldconfig
		
		rm -rf /tmp/curl_7_73_ubuntu*
	  EOH
	end
end

#Instalo paquete necesario para el binario icc_licenser
if node["platform"] == "ubuntu" then
	package "libssh-dev"
end

#Instalo ElasticSearch
include_recipe "elasticsearch::default"

#Exporters para Grafana
include_recipe "i6::grafana_exporters"
