
#
# ONLY RUN ONCE in the fist node
#

# FIXME when migrations are available we use them and forget about this stuff.
include_recipe "apt"

execute "packages update check" do
  command "ls"
  notifies :run, resources(:execute => "apt-get update"), :immediately
end

package "build-essential" do 
	options '--force-yes -o Dpkg::Options::="--force-confnew" '
end 

package "autoconf" do 
	options '--force-yes -o Dpkg::Options::="--force-confnew" '
end 

package "libtool" do 
	options '--force-yes -o Dpkg::Options::="--force-confnew" '
end 

# Paquete para firefox =======================================================
if node["platform"] == "debian" then
	package "firefox-esr" do
	  #version "52.9.0esr-1~deb8u1"
	  # force yes is a hack because I do not have a signed repo.
	  options '--force-yes -o Dpkg::Options::="--force-confnew" '
	end
end

if node["platform"] == "ubuntu" then
	package "firefox" do
	  #version "52.9.0esr-1~deb8u1"
	  # force yes is a hack because I do not have a signed repo.
	  options '--force-yes -o Dpkg::Options::="--force-confnew" '
	end
end

bash "download FirefoxSetup" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    wget -q http://#{node.inconcert.sources.server}/repo/FirefoxSetup.tar.bz2 -O /tmp/FirefoxSetup.tar.bz2
  EOH
end

bash "Install firefox from gz file" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    mkdir /opt/firefox
	tar xjf FirefoxSetup.tar.bz2 -C /opt/firefox/
  EOH
end

if node["platform"] == "debian" then
	bash "Link firefox on Debian" do
	  user    "root"
	  cwd     "/tmp"
	  code  <<-EOH
		mv /usr/lib/firefox-esr/firefox-esr /usr/lib/firefox-esr/firefox-esr_orig
		ln -s /opt/firefox/firefox/firefox /usr/lib/firefox-esr/firefox-esr
	  EOH
	end
end

if node["platform"] == "ubuntu" then
	bash "Link firefox on Ubuntu" do
	  user    "root"
	  cwd     "/tmp"
	  code  <<-EOH
		rm -f /usr/bin/firefox
		ln -s /opt/firefox/firefox/firefox /usr/bin/firefox
	  EOH
	end
end

# Paquete geckodriver ================================================================
geckodriverFile="geckodriver-v0.21.0-linux64.tar.gz"
if node["platform"] == "ubuntu" then
	geckodriverFile="geckodriver-v0.24.0-linux64.tar.gz"
end
bash "download geckodriver" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    wget -q http://#{node.inconcert.sources.server}/repo/#{geckodriverFile} -O /tmp/#{geckodriverFile}
  EOH
end

bash "Install geckodriver from gz file" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    sh -c 'tar -x geckodriver -zf #{geckodriverFile} -O > /usr/bin/geckodriver'
	chmod +x /usr/bin/geckodriver
  EOH
end

# Paquete python3 ================================================================
package "python3" do
  # force yes is a hack because I do not have a signed repo.
  options '--force-yes -o Dpkg::Options::="--force-confnew" '
end
package "python3-dev" do
  # force yes is a hack because I do not have a signed repo.
  options '--force-yes -o Dpkg::Options::="--force-confnew" '
end
package "python3-pip" do
  # force yes is a hack because I do not have a signed repo.
  options '--force-yes -o Dpkg::Options::="--force-confnew" '
end

#Seteo variables de entorno para que se descarguen los paquetes desde los repo locales.
bash "Set pip source repo" do
  user    "root"
  cwd     "root"
  code  <<-EOH
	echo [easy_install] > ~/.pydistutils.cfg
	echo index-url=http://#{node.inconcert.sources.server}/repo/pip3/ >> ~/.pydistutils.cfg
  EOH
end

# Install Packages with pip ================================================================
#pip3 install -v --upgrade --ignore-installed --index-url http://#{node.inconcert.sources.server}/repo/pip3/ pip
bash "Packages pip" do
  user    "root"
  cwd     "root"
  code  <<-EOH
	pip3 install -v --upgrade --index-url http://#{node.inconcert.sources.server}/repo/pip3/ pip
  EOH
end

bash "Packages setuptools" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --ignore-installed --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ setuptools
  EOH
end

bash "Packages pyOpenSSL" do
  user    "root"
  cwd     "root"
  code  <<-EOH
	easy_install pyOpenSSL
  EOH
end

bash "Packages redis" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ redis
  EOH
end

bash "Packages pydub" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ pydub
  EOH
end

bash "Packages cryptography" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ cryptography
  EOH
end

bash "Packages enum34" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ enum34
  EOH
end

bash "Packages python-magic" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ python-magic
  EOH
end

bash "Packages typing" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ typing
  EOH
end

bash "Packages numpy" do
  user    "root"
  cwd     "root"
  code  <<-EOH
	python3.4 -m pip install -v --trusted-host #{node.inconcert.sources.server} --upgrade --index-url http://#{node.inconcert.sources.server}/repo/pip3/ numpy
  EOH
  only_if { node["platform"] == "debian" }
end

bash "Packages numpy" do
  user    "root"
  cwd     "root"
  code  <<-EOH
	python3.6 -m pip install -v --trusted-host #{node.inconcert.sources.server} --upgrade --index-url http://#{node.inconcert.sources.server}/repo/pip3/ubuntu/ numpy
  EOH
  only_if { node["platform"] == "ubuntu" }
end

bash "Packages numpy" do
  user    "root"
  cwd     "root"
  code  <<-EOH
	python3.6 -m pip install -v --trusted-host #{node.inconcert.sources.server} --upgrade --index-url http://#{node.inconcert.sources.server}/repo/pip3/ubuntu/ protobuf
  EOH
  only_if { node["platform"] == "ubuntu" }
end

bash "Packages selenium" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ selenium
  EOH
end

bash "Packages python_dateutil" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ python_dateutil
  EOH
end

bash "Packages urllib3" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ urllib3
  EOH
end

bash "Packages idna" do
  user    "root"
  cwd     "root"
  code  <<-EOH
    pip3 install -v --upgrade --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ idna
  EOH
end

bash "download ebWhatsapp-Wrapper" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    wget -q http://#{node.inconcert.sources.server}/repo/WebWhatsapp-Wrapper.tar.gz -O /tmp/WebWhatsapp-Wrapper.tar.gz
  EOH
end

bash "Packages WebWhatsapp-Wrapper" do
  user    "root"
  cwd     "/tmp/"
  code  <<-EOH
	tar -xvzf WebWhatsapp-Wrapper.tar.gz
	cd webwhatsapp-wrapper
    pip3 install -v --trusted-host #{node.inconcert.sources.server} --index-url http://#{node.inconcert.sources.server}/repo/pip3/ ./ --upgrade
  EOH
end

bash "geckodriver permissions" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	chmod +x /usr/bin/geckodriver
  EOH
end
