#
# Cookbook Name:: keepalived
# Recipe:: default
#
# Copyright 2009, Opscode, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

package "keepalived" do
  action :install
end

bash "unistall keepalived init.d service" do
  user "root"
  code "update-rc.d -f keepalived remove"
end

directory "/etc/keepalived/sv" do
  owner "root"
  mode 0776
  recursive true
end

runit_service "keepalived" do
  action    :nothing
  directory "/etc/keepalived/sv"
  log       false
end

bash "reload keepalived" do
  user "root"
  code "sv hup keepalived"
  action :nothing
end

template "/etc/keepalived/keepalived.conf" do
  source "keepalived.conf.erb"
  owner "root"
  group "root"
  mode 0644
  notifies :restart, resources(:service => "keepalived")
end

bash "make ip_vs module load on boot" do
  user "root"
  cwd  "/"
  code "echo ip_vs >> /etc/modules"
  not_if "grep ip_vs /etc/modules"
end

bash "install ip_vs" do
  user "root"
  code "modprobe ip_vs"
  not_if "modprobe -l|grep ip_vs"
end


node.keepalived.extrafiles.each do |source, dest| 
  template dest do
    source source
    owner "root"
    group "root"
    mode 0644
    notifies :restart, resources(:service => "keepalived")
  end
end






