#
# Cookbook Name:: MySql
# Recipe:: default
#
# Copyright 2023, inConcert
#
# All rights reserved - Do Not Redistribute
#


#
# make helper methods available inside templates.
#
#class Erubis::Context
#	include Redis::Helpers
#end

class Erubis::Context
  include MySql::GenericHelpers
end

#Chequeos iniciales ====================================================
if not node["platform"] == "ubuntu" then
	raise "El Sistema Operativo debe ser UBUNTU"
end

bash "[MYSQL] Check previous version of MySQL" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	mysqlInstall=`dpkg -l | grep mysql-client-5.7`
	if [ ! "$mysqlInstall" == '' ]; then
		echo "Existe una version previa de MySQL instalada. Debe desinstalar esto antes de correr el instalador."
		exit 1
	fi
	mysqlInstall=`dpkg -l | grep mysql-client-core-5.7`
	if [ ! "$mysqlInstall" == '' ]; then
		echo "Existe una version previa de MySQL instalada. Debe desinstalar esto antes de correr el instalador."
		exit 1
	fi
  EOH
end

bash "[MYSQL] enviroment values" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	echo "Ubuntu #{node.lsb.release}"
  echo "Local IP: #{node.ipaddress}"
  EOH

end

if node.mysql.install_type == "standalone" then
	include_recipe "mysql::firewall"
end

#Descargo bundle de mySQL ====================================================
bash "[MYSQL] Download mysql-server_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb-bundle.tar" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	mkdir -p /tmp/mysql_packages
	cd /tmp/mysql_packages
	wget -q http://#{node.mysql.sources.server}/repo/mysql-server_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb-bundle.tar -O /tmp/mysql_packages//mysql-server_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb-bundle.tar
	touch /tmp/mysql_packages/.downloadOK
  EOH
  not_if "ls /tmp/mysql_packages/.downloadOK"
end


bash "[MYSQL] create mysql config directory" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    mkdir /etc/mysql
  EOH
  not_if "ls /etc/mysql"
end

bash "[MYSQL] unpack mysql-server_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb-bundle.tar" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	  cd /tmp/mysql_packages
	  tar -xvf /tmp/mysql_packages/mysql-server_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb-bundle.tar
  EOH
end


#Instalo MySQL Server ====================================================
package "libaio1"
package "libmecab2"
if node["platform_version"].start_with? "18" then
  package "libssl1.1" do
    action :upgrade
  end
end

template "/etc/mysql/my.cnf.fallback" do
  source "my.cnf.fallback.erb"
  owner  "root"
  group  "root"
  mode   "0644"
end

bash "[MYSQL] Install MYSQL_#{node.grafana.version_grafana} MySQL Client" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	cd /tmp/mysql_packages/
  rm -f /etc/mysql/mysql.cnf
	dpkg -i mysql-common_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb 
	dpkg -i mysql-community-client-plugins_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb
	dpkg -i mysql-community-server-core_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb
	dpkg -i mysql-community-client-core_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb
	dpkg -i mysql-community-client_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb
  EOH
end


bash "[MYSQL] Install MYSQL_#{node.grafana.version_grafana} MySQL Server" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
	cd /tmp/mysql_packages/
  export DEBIAN_FRONTEND=noninteractive
	dpkg -i mysql-client_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb
	dpkg -i mysql-community-server_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb
	dpkg -i mysql-cluster-community-server-debug_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb
	
	dpkg -i mysql-server_#{node.mysql.version_mysql}#{node.platform}#{node.platform_version}_amd64.deb
  EOH
end

if node.mysql.node_master.mysql_private_ip == node.ipaddress
  template "/etc/mysql/my.cnf" do
   source "my.cnf.erb"
   owner  "root"
   group  "root"
   mode   "0644"
   variables(
     server_id: node.mysql.node_master.server_id
   )
 end
else
  node.mysql.node_master.node_slave.map do |n|
   if n['mysql_private_ip']==node.ipaddress
     template "/etc/mysql/my.cnf" do
       source "my.cnf.erb"
       owner  "root"
       group  "root"
       mode   "0644"
       variables(
         server_id: n['server_id']
       )
     end
    end
  end
end

bash "[MySQL] Restart service" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    systemctl restart mysql
  EOH
end

#Instalar Exporters para Grafana ====================================================
bash "[NODE_EXPLORER] Download Node Exporter" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    wget http://#{node.mysql.sources.server}/repo/node_exporter-1.3.1.linux-amd64.tar.gz
  EOH
end

bash "[NODE_EXPLORER] Install Node Exporter" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    mkdir /etc/prometheus_exporters
    sudo tar zxf node_exporter-1.3.1.linux-amd64.tar.gz -C /etc/prometheus_exporters
  EOH
end

cookbook_file "/etc/systemd/system/node_exporter.service" do
  source "node_exporter.service"
  owner   "root"
  group   "root"
  mode    "0644"
end

bash "[NODE_EXPLORER] Enable and start exporter" do
  cwd "/etc/init.d"
  code <<-EOC
    systemctl daemon-reload
    systemctl enable node_exporter
	  systemctl start node_exporter
  EOC
end
#

#Instalar Mysql Exporter  ====================================================
bash "[MYSQLD_EXPLORER] Install Mysqld Exporter" do
  user    "root"
  cwd     "/tmp"
  code  <<-EOH
    mkdir -p /etc/prometheus_exporters/mysqld_exporter
  EOH
end

bash "[MYSQLD_EXPLORER] Download binary file" do
  cwd "/etc/init.d"
  code <<-EOC
    wget -q http://#{node.mysql.sources.server}/repo/mysqld_exporter_#{node.lsb.codename} -O  /etc/prometheus_exporters/mysqld_exporter/mysqld_exporter
  EOC
  not_if "ls /etc/prometheus_exporters/mysqld_exporter/mysqld_exporter"
end

cookbook_file "/etc/prometheus_exporters/mysqld_exporter/my.cnf" do
  source "my.cnf"
  owner   "root"
  group   "root"
  mode    "0644"
end

cookbook_file "/etc/systemd/system/mysql_exporter.service" do
  source "mysql_exporter.service"
  owner   "root"
  group   "root"
  mode    "0644"
end


bash "[MYSQLD_EXPLORER] Enable and start exporter" do
  cwd "/etc/init.d"
  code <<-EOC
    chmod 744 /etc/prometheus_exporters/mysqld_exporter/mysqld_exporter
    systemctl daemon-reload
    systemctl enable mysql_exporter
    systemctl start mysql_exporter
  EOC
end

bash "[MYSQLD_EXPLORER] create users exporter" do
  cwd "/"
  code <<-EOC
    mysql -e "CREATE USER 'exporter'@'127.0.0.1' IDENTIFIED BY 'exporter' WITH MAX_USER_CONNECTIONS 5"
    mysql -e "GRANT PROCESS, REPLICATION CLIENT, SELECT ON *.* TO 'exporter'@'127.0.0.1'"
  EOC
end

# Preparar replicacion
if node.mysql.node_master.mysql_private_ip == node.ipaddress # configuraciones en el nodo maestro
  node.mysql.node_master.node_slave.map do |n|
    bash "[MySQL Replication] create users and grant replication" do
      cwd "/"
      code <<-EOC
        mysql -e "create user '#{node.mysql.user_replication}'@'#{n['mysql_private_ip']}' identified by '#{node.mysql.password_replication}'"
        mysql -e "grant replication slave on *.* to'#{node.mysql.user_replication}'@'#{n['mysql_private_ip']}'"
      EOC
    end
    bash "[MySQL Replication] create users and grant replication" do
      cwd "/"
      code <<-EOC
        mysql -e "create user 'config'@'#{n['mysql_private_ip']}' identified by '#{node.mysql.password_replication}'"
        mysql -e "GRANT ALL PRIVILEGES ON *.*  TO  'config'@'#{n['mysql_private_ip']}'"
        mysql -e "flush privileges"
      EOC
    end
  end
else  #configuraciones en los nodos slave
  node.mysql.node_master.node_slave.map do |n|
    if n['mysql_private_ip']==node.ipaddress
     bash "[MySQL Replication] create users and grant replication" do
      cwd "/"
      code <<-EOC
        master_status=$(mysql -h#{node.mysql.node_master.mysql_private_ip}  -uconfig  -p#{node.mysql.password_replication} --silent --skip-column-names  -e "show master status")
        IFS=$'\t'
        tmp=($master_status)
        file="${tmp[0]}"
        position="${tmp[1]}"
        mysql -e "stop slave"
        mysql -e "CHANGE MASTER TO MASTER_HOST = '#{node.mysql.node_master.mysql_private_ip}', MASTER_USER = '#{node.mysql.user_replication}', MASTER_PASSWORD = '#{node.mysql.password_replication}', MASTER_LOG_FILE = '${file}', MASTER_LOG_POS = ${position}"
        mysql -e "START SLAVE"
        mysql -e "START REPLICA"
        echo 'Use -- mysql -e "SHOW REPLICA STATUS\\G" -- to verify replication'  
        mysql -e 'SHOW REPLICA STATUS\\G'
      EOC
      end
    end
  end
end