#
# Cookbook Name:: redis
# Recipe:: default
#
# Copyright 2012, inConcert
#
# All rights reserved - Do Not Redistribute
#


#
# make helper methods available inside templates.
#
class Erubis::Context
	include Redis::Helpers
end

include_recipe "runit"

# Pin Redis version at 2.8.x
cookbook_file "/etc/apt/preferences.d/pin-redis" do
	source "pin-redis"
	mode "644"
end

package "redis" do
	version node.redis.version
	options '--force-yes -o Dpkg::Options::="--force-confold" -o Dpkg::Options::="--force-confmiss"'   # this is a hack because I do not have a signed repo.
end

include_recipe "redis::sysctl"

node.default['redis']['servers'].each do |k,v|
	instance_name =
				if k == "default" then
					 ""
				 else
					 ".#{k}"
				 end

	# we want to get version without build number
	redis_version = node.redis.version.split("-")[0]
	#
	redis_version_split = redis_version.split(".")
	redis_version_major_minor = redis_version_split[0] + "." + redis_version_split[1]
	v['port'] = node.redis['servers'][k].port
	if node.redis['servers'][k]['sentinel_port'] then
		v['sentinel_port'] = node.redis['servers'][k]['sentinel_port']
	end
	exists_sentinel = false
	is_sentinel = false
	if node.redis['servers'][k]['sentinel'] and node.redis['servers'][k]['sentinel'].length() > 0 then
		exists_sentinel = true
		bash "REDIS: Exists redis sentinel for #{k}" do
			user "root"
			code ""
			action :nothing
		end
		node.redis['servers'][k]['sentinel'].map do |sentinel_node|
			if sentinel_node['id'] == node.hostname then
				bash "REDIS: #{node.hostname} node is redis sentinel" do
					user "root"
					code ""
					action :nothing
				end
				is_sentinel = true
			end
		end
	end
	if is_sentinel then
		template "/etc/redis/sentinel#{instance_name}.conf" do
			source "sentinel.conf.erb"
			owner "redis"
			variables({ :redis_instance => v, :redis_instance_name => k })
			not_if "ls /etc/redis/.sentinel#{instance_name}.#{node.redis.version}.bootstrapped"
		end
		bash "" do
			user "root"
			code "touch /etc/redis/.sentinel#{instance_name}.#{node.redis.version}.bootstrapped"
		end
	end
	
	if not exists_sentinel then
		template "/etc/redis/redis#{instance_name}.conf" do
			source "redis-#{redis_version_major_minor}.x.conf.erb"
			owner "redis"
			variables({ :redis_instance => v, :redis_instance_name => k })
		end
	else
		template "/etc/redis/redis#{instance_name}.conf" do
			source "redis-#{redis_version_major_minor}.x.conf.erb"
			owner "redis"
			variables({ :redis_instance => v, :redis_instance_name => k })
			not_if "ls /etc/redis/.redis#{instance_name}.#{node.redis.version}.bootstrapped"
		end
		bash "" do
			user "root"
			code "touch /etc/redis/.redis#{instance_name}.#{node.redis.version}.bootstrapped"
		end
	end
	
	# run redis backup daily or hourly
	redis_instance_name_tmp = instance_name.empty? ? "redis_default" : "redis_#{k}"
	if k == "outboundengine" then
		redis_file_conf = "/etc/redis/redis#{instance_name}.conf"
		template "/etc/cron.daily/daily_backup_#{redis_instance_name_tmp}" do
		  source "daily_backup_redis"
		  owner "root"
		  group "root"
		  mode  0744
		  variables({ :redis_file_conf => redis_file_conf,  :redis_instance => v, :redis_instance_name => k })
		end
		#Borro el backup por hora que se puede haber generado en builds anteriores.
		bash "" do
			user "root"
			code "rm -f /etc/cron.hourly/hourly_backup_redis_outboundengine"
		end
	else
		redis_file_conf = "/etc/redis/redis#{instance_name}.conf"
		template "/etc/cron.hourly/hourly_backup_#{redis_instance_name_tmp}" do
		  source "hourly_backup_redis"
		  owner "root"
		  group "root"
		  mode  0744
		  variables({ :redis_file_conf => redis_file_conf,  :redis_instance => v, :redis_instance_name => k })
		end
	end
	#Reinicio cron
	bash "" do
		user "root"
		code "/etc/init.d/cron restart"
	end
	
	#
	# create services
	#
	redis_service_name = instance_name.empty? ? "redis" : "redis-#{k}"
	if redis_service_name != "redis" then
		directory "/etc/redis/sv/#{redis_service_name}" do
			owner "root"
			mode 0776
			recursive true
		end
		runit_service "#{redis_service_name}" do
			template_name "default"
			action    :nothing
			directory "/etc/redis/sv"
			log       false
			options  :redis_instance_name => instance_name
		end
		bash "reload #{redis_service_name}" do
			user "root"
			code "sv hup #{redis_service_name}"
			action :nothing
		end
	else
		template "/etc/redis/sv/redis/run" do
			owner "root"
			group "root"
			mode 0755
			source "sv-default-run.erb"
			variables :options => { :redis_instance_name => instance_name}
		end
	end

	#
	# create Sentinel services
	#	
	redis_service_name = instance_name.empty? ? "redis-sentinel" : "redis-sentinel-#{k}"
	if is_sentinel then
		directory "/etc/redis/sv/#{redis_service_name}" do
			owner "root"
			mode 0776
			recursive true
		end
		runit_service "#{redis_service_name}" do
			template_name "sentinel-default"
			action    :nothing
			directory "/etc/redis/sv"
			log       false
			options  :redis_instance_name => instance_name
		end
		bash "reload #{redis_service_name}" do
			user "root"
			code "sv hup #{redis_service_name}"
			action :nothing
		end
	else
		#Si no es un sentinel borro un posible servicio existente
		bash "stop service #{redis_service_name}" do
			user "root"
			code "sv stop #{redis_service_name}"
			only_if "ls /etc/service/#{redis_service_name}"
		end
		bash "remove service #{redis_service_name}" do
			user "root"
			code  <<-EOH
				/usr/sbin/update-service --remove /etc/redis/sv/#{redis_service_name}
			EOH
			only_if "ls /etc/service/#{redis_service_name}"
		end
	end
end
